/**
 * Copyright (c) 2004 Logos GmbH Rostock (www.logos-ib.de).
 * All rights reserved.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL  LOGOS GMBH  OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package de.inovat.kex.bast.common;

//~ NICHT JDK IMPORTE =========================================================

import de.bsvrz.dav.daf.main.*;
import de.bsvrz.sys.funclib.commandLineArgs.ArgumentList;
import de.bsvrz.sys.funclib.debug.Debug;

//~ KLASSEN ===================================================================

/**
 * <p>Klasse, die eine ClientDavConnection global zur Verf&uuml;gung stellt.</p>
 * <p>
 * Klasse wurde überschrieben: ClientDavInterface wird in der Klasse
 * {@link de.bsvrz.sys.funclib.application.StandardApplication} erstellt und
 * über die Methode {@link #initDav(de.bsvrz.dav.daf.main.ClientDavInterface)}
 * hier initialisiert.
 * <p>
 * @author LOGOS GmbH Rostock (www.logos-ib.de)
 * @author Ralf Trier
 * @version $Id: LogosClientDavConnection.java 442 2008-11-10 06:59:32Z Schawaller $
 */
public class NetzClientDavConnection {

    /** DaV für den Archiv oder null, falls aktuelle DaV für die Archiv-Anfragen verwendet werden soll. */
    private static ClientDavInterface _davArchiv                     = null;
    private static boolean            _davArchivAlternativeVerwenden = false;

    /** Datenverteilerverbindung */
    private static ClientDavInterface _dav;

    //~ METHODEN ==============================================================

    /**
     * Erstelle eine DaV-Verbindung.
     *
     * @param davHost Host.
     * @param davBenutzer Benutzer.
     * @param davPasswort Passwort.
     *
     * @return DaV.
     */
    private static ClientDavInterface erstelleDavVerbindung(String davHost, String davBenutzer, String davPasswort) {
        ClientDavInterface dav;

        try {

            // Argumente
            ArgumentList        argList = new ArgumentList(new String[] { davHost });
            ClientDavParameters cdp     = new ClientDavParameters(argList);

            // DAV-Verbindung
            dav = new ClientDavConnection(cdp);
            dav.connect();
            dav.login(davBenutzer, davPasswort);
        }
        catch (Exception e) {
            dav = null;

            try {
                Debug.getLogger().error(String.format("DaV-Verbindung <%s><%s><%s> kann nicht hergestellt werden!", davHost, davBenutzer, davPasswort), e);
            }
            catch (Exception ignored) {}
        }

        return dav;
    }

    //~ GET METHODEN ==========================================================

    /**
     * gibt die letzte ClientDavConnection zurück oder null.
     *
     * @return ClientDavConnection
     */
    public static ClientDavInterface getInstance() {
        return _dav;
    }

    /**
     * gibt die Archiv-DaV zurück oder null.
     *
     * @return ClientDavConnection
     *
     * s.{@link de.inovat.kex.bast.exporter.BAStExportSteuerung#_davArchivDatenverteiler}
     */
    public static ClientDavInterface getInstanceArchiv() {
        return (_davArchivAlternativeVerwenden) ? _davArchiv : _dav;
    }

    //~ METHODEN ==============================================================

    /**
     * Initialisiere ClientDavInterface.
     *
     * @param dav ClientDavInterface.
     */
    public static void initDav(ClientDavInterface dav) {
        _dav = dav;
    }

    /**
     * Initialisiere ClientDavInterface.
     *
     * @param davHost Host.
     * @param davBenutzer Benutzer.
     * @param davPasswort Passwort.
     */
    public static void initDavArchiv(String davHost, String davBenutzer, String davPasswort) {
        _davArchiv                     = erstelleDavVerbindung(davHost, davBenutzer, davPasswort);
        _davArchivAlternativeVerwenden = true;
    }
}


//~Formatiert mit 'inovat Kodierkonvention' am 15.09.16
