/**
 * Copyright (c) 2004 Logos GmbH Rostock (www.logos-ib.de).
 * All rights reserved.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL  LOGOS GMBH  OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package de.inovat.kex.bast.common.datawrapper;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.config.IntegerValueRange;
import de.bsvrz.dav.daf.main.config.IntegerValueState;

/*
 * Revision 1.4  2008/09/04 12:52:13  Logosdev
 * Prüfungen mit isState() und isNumber() von getUnscaledValue() auf getScaledValue() umgestellt, da 'Unscaled Values' für isNumber() immer true liefern (weil auch Zustände als Zahl ausgelesen werden können, was zu Fehlern beim BASt-Export in der DAUZ ST führte)
 */

/**
 * kapselt die AttributListe atl.verkehrsStaerkeStunde oder atg.langZeitDatenFahrzeugAnzahl.
 * 
 * @author LOGOS GmbH Rostock (www.logos-ib.de)
 * @author J. Schawaller
 * @version $Id: AtlVerkehrsStaerkeStunde.java 442 2008-11-10 06:59:32Z Schawaller $
 */
public class AtlVerkehrsStaerkeStunde {

  private Data _Data;

  public static final String ATTR_NAME_WERT = "Wert";
  public static final String ATTR_NAME_STATUS = "Status";
  public static final String ATTR_NAME_GUETE = "Güte";

  public static final String ZUSTAND_WERT_NICHT_ERMITTELBAR = "nicht ermittelbar";
  public static final String ZUSTAND_WERT_FEHLERHAFT = "fehlerhaft";
  public static final String ZUSTAND_WERT_NICHT_ERMITTELBAR_FEHLERHAFT = "nicht ermittelbar/fehlerhaft";

  protected boolean _hasState = true;

  public AtlVerkehrsStaerkeStunde(Data data) {
    _Data = data;
    _hasState = "atl.verkehrsStärkeStunde".equals(data.getAttributeType().getPid());
  }

  public void doInitialize() {
    new AtlStatus(_Data.getItem("Status")).doInitialize();
    new AtlGuete(_Data.getItem("Güte")).doInitialize();
    _Data.getScaledValue("Wert").setText(ZUSTAND_WERT_NICHT_ERMITTELBAR);
  }

  public IntegerValueRange getRangeOfVerkehrsStaerkeStunde() {
    return ValueRangeHashtable.getInstance(this._Data).get("att.verkehrsStärkeStunde");
  }
  public int getWert() {
    return _Data.getScaledValue("Wert").intValue();
  }

  public void setWert (int wert) {
    try {
      if (wert < 0) {
        _Data.getUnscaledValue("Wert").set(wert);
      }
      else {
        _Data.getScaledValue("Wert").set(wert);
      }
    }
    catch (IllegalArgumentException ex) {
      long lMaxValue = getRangeOfVerkehrsStaerkeStunde().getMaximum();
      long lMinValue = getRangeOfVerkehrsStaerkeStunde().getMinimum();
      throw new IllegalArgumentException("Wert für VerkehrsStärkeStunde konnte nicht auf " +
                                         wert + " gesetzt werden. (Min/Max:" +
                                         lMinValue+"/"+lMaxValue + ")");
    }
  }
//      _Data.getScaledValue("Wert").set(wert);
//  }
  public IntegerValueState getZustandWert() {
    return _Data.getScaledValue("Wert").getState();
  }

  public void setZustandWert (IntegerValueState integerValueState) {
      _Data.getScaledValue("Wert").setState(integerValueState);
  }
  public void setZustandWert (String zustandsName) {
      _Data.getScaledValue("Wert").setText(zustandsName);
  }

  // Also: _Data.getUnscaledValue("Wert").isNumber() liefert IMMER true, da sich die Zustände ja als Zahl abfragen lassen
  //       _Data.getUnscaledValue("Wert").isState()  liefert true falls Zustand
  // eigentlich geht es aber um _Data.getScaledValue("Wert") also:
  //       _Data.getScaledValue("Wert").isNumber() liefert true, wenn es wirklich ein Wert ist
  //       _Data.getScaledValue("Wert").isState()  liefert true, wenn es ein Zustand ist

  public boolean isState() {
//    return !_Data.getUnscaledValue("Wert").isNumber();
    return _Data.getScaledValue("Wert").isState();
  }

  public boolean isNumber() {
//    return _Data.getUnscaledValue("Wert").isNumber();
    return _Data.getScaledValue("Wert").isNumber();
  }

  /**
   * liefert AtlStatus-Instance zur&uuml;ck, wenn das gekapselte Data vom Typ
   * atl.verkehrsStaerkeStunde ist und einen Status besitzt (sonst null).
   * @return AtlStatus
   */
  public AtlStatus  getStatus() {
    if (_hasState) {
      return new AtlStatus(_Data.getItem("Status"));
    }
    return null;
  }

  /**
   * liefert true zur&uuml;ck, wenn das gekapselte Data vom Typ
   * atl.verkehrsStaerkeStunde ist und
   * einen Status besitzt.
   *
   * @return boolean
   */
  public boolean hasState() {
    return _hasState;
  }

  public AtlGuete  getGuete() {
      return new AtlGuete(_Data.getItem("Güte"));
  }

  public Data getData() {
      return _Data;
  }

  /**
   * gibt den Wert mit einem Pr&uuml;fkennzeichen zur&uuml;ck.
   *
   * <table border=1><caption>Prüfzeichen</caption>
   * <tr><td>-</td><td>korrekter regelm&auml;ssiger Wert</td><td>&nbsp;</td></tr>
   * <tr><td>u</td><td>fehlender Wert (Ausfall)</td><td>wenn Status == nicht ermittelbar, Wert:=0</td></tr>
   * <tr><td>d</td><td>fehlerhafter Wert</td><td>wenn G&uuml;te &lt;&gt;1, Wert wird aus der Attributegruppe &uuml;bernommen.</td></tr>
   * </table>
   *
   * @return String
   */
  public String getValueAsBastString() {

    String retValue;
    if (this.isState()) {
      retValue = "0a";
    }
    else {
      AtlGuete guete = this.getGuete();
      StringBuffer sb = new StringBuffer(5);
      sb.append(this.getWert());
      // if (guete.isNumber() && (guete.getIndex()>0.9999999)) {
      if (guete.isNumber()) {
        sb.append('-');
      }
      else {
        sb.append('d');
      }
      retValue = sb.toString();

    }
    return retValue;
  }





}

