/**
 * Copyright (c) 2004-2008 Logos GmbH Rostock (www.logos-ib.de).
 * All rights reserved.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL  LOGOS GMBH  OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package de.inovat.kex.bast.common.objectwrapper;

/**
 * History:
 * 2008-10-17  Beseitung der Abhängigkeiten zum KEX-TLS Projekt durch Löschung
 *             der Klassenreferenzen und Verwendung von Stringkonstanten,
 *             Einführung der Superklasse FGdefaultType
 * @since Revision 1.3  2007/10/15 13:14:32  Strey
 */

import de.inovat.kex.bast.common.constants.ConfigurationConstants.ATG;

import java.util.Hashtable;

/**
 * <p>Kapselt die unterschiedlichen Typen der FG03 (Lufttemperatur,
 * Fahrbahnoberfl&auml;chentemperatur, Windgeschwindigkeit usw).</p>
 * 
 * @author LOGOS GmbH Rostock (www.logos-ib.de)
 * @author Ralf Trier
 * @version $Id: FG03Type.java 442 2008-11-10 06:59:32Z Schawaller $
 */
public class FG03Type extends FGdefaultType {

  private static Hashtable<String, FG03Type> __MappingTyp = new Hashtable<String, FG03Type>(30);
  private static boolean __NotInitialized = true;

  /**
   * <p>Initialisiert die lokalen Variablen und fügt das erzeugte Objekt der
   * Mappingtabelle hinzu:</p>
   * 
   * @param umsetzungsModul Klassenname/Konfigurationsangabe für Umsetzungsmodul
   * @param name Bezeichnung des Sensorwerts z.B. "Lufttemperatur"
   * @param abbreviation Abkürzung des Sensorwerts z.B. "LT"
   * @param type Typ
   * @param atgPid Attributgruppe, mit der das Umsetzungsmodul die Fachdaten
   *   liefert, z.B. "atg.tlsUfdErgebnisMeldungLuftTemperaturLT" 
   * @param isFahrstreifenSensor boolean <code>true</code> falls der Sensorwert
   *   fahrbahnbezogen ist, ansonsten <code>false</code>
   */
  public FG03Type(String umsetzungsModul, String name, String abbreviation, int type, String atgPid, boolean isFahrstreifenSensor) {
  	super(umsetzungsModul, name, abbreviation, type, atgPid, isFahrstreifenSensor);
    __MappingTyp.put(_UmsetzungsModul, this);
  }

  /**
   * <p>Liefert das Typ-Objekt zum angegebenen Umsetungsmodul (Java Klasse).</p>
   *
   * @param umsetzungsModul String Name des Umsetzungsmoduls (Java Klassenname)
   * @return FG03Type
   */
  public static FG03Type getInstance(String umsetzungsModul) {
    if (__NotInitialized) {
      init();
    }
    return __MappingTyp.get(umsetzungsModul);
  }

  /**
   * <p>Registrierung bekannter Standardtypen aus der KEX-TLS</p>
   */
  private synchronized static void init() {
    if (__NotInitialized) {
    	// Mit der Package Umstrukturierung zwischen Version 3.2 und 3.4 haben
    	// sich die Klassennamen der TLS-Umsetzungsmodule geändert. In den
    	// Konfigurationen können aber weiterhin auch die alten Namen verwendet
    	// werden. Deshalb müssen jeweils 2 Bezeichner unterstützt werden.
    	for (String praefix : new String[] {"de.bsvrz.kex.tls.osi7.conversion.",
    			                                         "kex.tls.osi7.conversion."} )
    	{
    		new FG03Type(praefix+"Fg003Typ048", "Lufttemperatur", "LT", 48, ATG.TLS_UFD_ERGEBNISMELDUNG_LUFTTEMPERATUR, false);
    		new FG03Type(praefix+"Fg003Typ049", "Fahrbahnoberflächentemperatur", "FBT", 49, ATG.TLS_UFD_ERGEBNISMELDUNG_FAHRBAHNOBERFLAECHENTEMPERATUR, true);
    		new FG03Type(praefix+"Fg003Typ052", "Restsalz", "RS", 52, ATG.TLS_UFD_ERGEBNISMELDUNG_RESTSALZ, true);
    		new FG03Type(praefix+"Fg003Typ051", "Zustand Fahrbahnoberfläche", "FBZ", 51, ATG.TLS_UFD_ERGEBNISMELDUNG_FAHRBAHNOBERFLAECHE, true);
    		new FG03Type(praefix+"Fg003Typ053", "Niederschlagsintensität", "NI", 53, ATG.TLS_UFD_ERGEBNISMELDUNG_NIEDERSCHLAGSINTENSITAET, false);
    		new FG03Type(praefix+"Fg003Typ055", "Relative Luftfeuchte", "RLF", 55, ATG.TLS_UFD_ERGEBNISMELDUNG_RELATIVE_LUFTFEUCHTE, false);
    		new FG03Type(praefix+"Fg003Typ056", "Windrichtung", "WR", 56, ATG.TLS_UFD_ERGEBNISMELDUNG_WINDRICHTUNG, false);
    		new FG03Type(praefix+"Fg003Typ057", "Windgeschwindigkeit (Mittel)", "WGM", 57, ATG.TLS_UFD_ERGEBNISMELDUNG_WINDGESCHWINDIGKEIT_MITTEL, false);
    		new FG03Type(praefix+"Fg003Typ058", "Schneehöhe", "SH", 58, ATG.TLS_UFD_ERGEBNISMELDUNG_SCHNEEHOEHE, false);
    		new FG03Type(praefix+"Fg003Typ060", "Sichtweite", "SW", 60, ATG.TLS_UFD_ERGEBNISMELDUNG_SICHTWEITE, false);
    		new FG03Type(praefix+"Fg003Typ063", "Niederschlagsart", "NSA", 63, ATG.TLS_UFD_ERGEBNISMELDUNG_NIEDERSCHLAG, false);
    		new FG03Type(praefix+"Fg003Typ064", "Windgeschwindigkeit (Spitze)", "WGS", 64, ATG.TLS_UFD_ERGEBNISMELDUNG_WINDGESCWHWINDIGKEIT_SPITZE, false);
    		new FG03Type(praefix+"Fg003Typ065", "Gefriertemperatur", "GT", 65, ATG.TLS_UFD_ERGEBNISMELDUNG_GEFRIERPUNKTTEMPERATUR, true);
    		new FG03Type(praefix+"Fg003Typ066", "Taupunkttemperatur", "TPT", 66, ATG.TLS_UFD_ERGEBNISMELDUNG_TAUPUNKTTEMPERATUR, false);
    		new FG03Type(praefix+"Fg003Typ067", "Tiefentemperatur (7 cm)", "TT 1", 67, ATG.TLS_UFD_ERGEBNISMELDUNG_TEMPERATUR_IN_TIEFE1, true);
    		new FG03Type(praefix+"Fg003Typ069", "Tiefentemperatur (30 cm)", "TT 3", 69, ATG.TLS_UFD_ERGEBNISMELDUNG_TEMPERATUR_IN_TIEFE2, true);
    		new FG03Type(praefix+"Fg003Typ070", "Zustand Fahrbahnoberfläche", "FBZ", 70, ATG.TLS_UFD_ERGEBNISMELDUNG_ZUSTAND_DER_FAHRBAHNOBERFLAECHE, true);
    		new FG03Type(praefix+"Fg003Typ071", "Niederschlagsart", "NS", 71, ATG.TLS_UFD_ERGEBNISMELDUNG_NIEDERSCHLAGSART, false);
    		new FG03Type(praefix+"Fg003Typ072", "Wasserfilmdicke", "WFD", 72, ATG.TLS_UFD_ERGEBNISMELDUNG_WASSERFILMDICKE, true);

    		// MicKS, herstellerspezifische Erweiterung - KEx-TLS Kernsoftware
    		new FG03Type(praefix+"mv.Fg003Typ182MvMicKS", "Zeitreserve Glätte", "ZRG", 182, ATG.TLS_UFD_ERGEBNISMELDUNG_FAHRBAHNGLAETTE_MICKS, true);
    		new FG03Type(praefix+"mv.Fg003Typ132MvMicKS", "Fahrbahnglätte", "FBG", 122,     				ATG.TLS_UFD_ERGEBNISMELDUNG_ZEITRESERVE_GLAETTE_MICKS, true);

    		// Vaisala, herstellerspezifische Erweiterung, KEx-TLS Kernsoftware
    		new FG03Type(praefix+"mv.Fg003Typ187MvAntBosch", "Zeitreserve Glätte", "ZRG", 187, ATG.TLS_UFD_ERGEBNISMELDUNG_ZEITRESERVE_GLAETTE, true);
    		new FG03Type(praefix+"mv.Fg003Typ189Mv", "Zustand Fahrbahnoberfläche", "FBZ", 189,     				ATG.TLS_UFD_ERGEBNISMELDUNG_FBZ_VAISALA, true);

    		// Vaisala, herstellerspezifische Erweiterung, nur KEx-TLS M-V
    		new FG03Type(praefix+"mv.Fg003Typ130MvVaisala", "Niederschlagssumme (Vaisala)", "NSS", 130, ATG.TLS_UFD_ERGEBNISMELDUNG_NIEDERSCHLAGSUMME_VAISALA, false);
    		new FG03Type(praefix+"mv.Fg003Typ131MvVaisala", "Menge Enteisungsmittel (Vaisala)", "GE", 131, ATG.TLS_UFD_ERGEBNISMELDUNG_MENGE_ENTEISUNGSMITTEL_VAISALA, true);
    		new FG03Type(praefix+"mv.Fg003Typ132MvVaisala", "NWS Code (Vaisala)", "NWS", 132, ATG.TLS_UFD_ERGEBNISMELDUNG_NWS_CODE_VAISALA, false);
    		new FG03Type(praefix+"mv.Fg003Typ133MvVaisala", "Interner Systemstatus (Vaisala)", "FDS", 133, ATG.TLS_UFD_ERGEBNISMELDUNG_SYSTEMSTATUS_VAISALA, false);
    		new FG03Type(praefix+"mv.Fg003Typ134MvVaisala", "WMO Code (Vaisala)", "WMO", 134, ATG.TLS_UFD_ERGEBNISMELDUNG_WMO_CODE_VAISALA, false);
    		new FG03Type(praefix+"mv.Fg003Typ65LiquidusMvVaisala", "Liquidustemperatur (Vaisala)", "LGT", 65, ATG.TLS_UFD_ERGEBNISMELDUNG_LIQUIDUSTEMPERATUR_VAISALA, true);
    	}
    }
    __NotInitialized = false;
  }
}
