/**
 * Copyright (c) 2004-2008 Logos GmbH Rostock (www.logos-ib.de).
 * All rights reserved.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL  LOGOS GMBH  OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package de.inovat.kex.bast.common.objectwrapper;

/*
 * History:
 * 2008-10-17  Beseitung der Abhängigkeiten zum KEX-TLS Projekt durch Löschung
 *             der Klassenreferenzen und Verwendung von Stringkonstanten,
 *             Einführung der Superklasse FGdefaultType
 * @since Revision 1.1  2007/08/02 11:31:21  Trier
 */


import java.util.Hashtable;

import de.inovat.kex.bast.common.constants.ConfigurationConstants.ATG;

/**
 * <p>Mit dem Objekt wird die Beziehung zwischen einem TLS "Umsetzungsmodul"
 * und der von ihm gelieferten "Attributgruppe" für den Sensorwert hergestellt.
 * FG06Type ist die FG-6 spezifische Implementierung</p>
 * 
 * @author LOGOS GmbH Rostock (www.logos-ib.de)
 * @author Ralf Trier
 * @version $Id: FG06Type.java 442 2008-11-10 06:59:32Z Schawaller $
 */
public class FG06Type extends FGdefaultType {

  private static boolean __NotInitialized = true;
  private static Hashtable<String, FG06Type> __MappingTyp = new Hashtable<String, FG06Type>(30);

  /**
   * <p>Initialisiert die lokalen Variablen und fügt das erzeugte Objekt der
   * Mappingtabelle hinzu:</p>
   * 
   * @param umsetzungsModul Klassenname/Konfigurationsangabe für Umsetzungsmodul
   * @param name Bezeichnung des Sensorwerts z.B. "Lufttemperatur"
   * @param abbreviation Abkürzung des Sensorwerts z.B. "LT"
   * @param type Typ
   * @param atgPid Attributgruppe, mit der das Umsetzungsmodul die Fachdaten
   *   liefert, z.B. "atg.tlsUfdErgebnisMeldungLuftTemperaturLT" 
   */
  public FG06Type(String umsetzungsModul, String name, String abbreviation, int type, String atgPid ) {
  	super(umsetzungsModul, name, abbreviation, type, atgPid, false);
    __MappingTyp.put(_UmsetzungsModul, this);
  }

  /**
   * <p>Liefert das Typ-Objekt zum angegebenen Umsetungsmodul (Java Klasse).</p>
   *
   * @param umsetzungsModul String Name des Umsetzungsmoduls (Java Klassenname)
   * @return FG06Type
   */
  public static FG06Type getInstance(String umsetzungsModul) {
    if (__NotInitialized) {
      init();
    }
    return __MappingTyp.get(umsetzungsModul);
  }
  
  /**
   * <p>Registrierung bekannter Standardtypen aus der KEX-TLS</p>
   */
  private synchronized static void init() {
    if (__NotInitialized) {
    	// Mit der Package Umstrukturierung zwischen Version 3.2 und 3.4 haben
    	// sich die Klassennamen der TLS-Umsetzungsmodule geändert. In den
    	// Konfigurationen können aber weiterhin auch die alten Namen verwendet
    	// werden. Deshalb müssen jeweils 2 Bezeichner unterstützt werden.
    	for (String praefix : new String[] {"de.bsvrz.kex.tls.osi7.conversion.",
    			                                         "kex.tls.osi7.conversion."} )
    	{
	  		new FG06Type(praefix+"Fg006Typ048", "Türkontakt", "Tür", 48, ATG.TLS_VLT_TUERKONTAKT);
	  		new FG06Type(praefix+"Fg006Typ049", "Temperaturüberwachung", "Temp", 55, ATG.TLS_VLT_TEMPERATURUEBERWACHUNG);
    		new FG06Type(praefix+"Fg006Typ050", "Licht", "Licht", 58, ATG.TLS_VLT_LICHT);
    		new FG06Type(praefix+"Fg006Typ051", "Stromversorgung", "Strom", 52, ATG.TLS_VLT_STROMVERSORGUNG);
    		new FG06Type(praefix+"Fg006Typ052", "Heizung", "Heizung", 60, ATG.TLS_VLT_HEIZUNG);
    		new FG06Type(praefix+"Fg006Typ053", "Lüftung", "Lüftung", 56, ATG.TLS_VLT_LUEFTUNG);
    		new FG06Type(praefix+"Fg006Typ054", "Überspannungsschutz", "Spannung", 53, ATG.TLS_VLT_UEBERSPANNUNGSSCHUTZ);
    		new FG06Type(praefix+"Fg006Typ055", "Diebstahl Vandalismus", "DiebVanadal", 49, ATG.TLS_VLE_DIEBSTAHL_VANDALISMUS_SCHUTZ);
    		new FG06Type(praefix+"Fg006Typ056", "Fernüberwachung von Solaranlagen", "Solar", 57, ATG.TLS_VLT_FERNUEBERWACHUNG_SOLAR_ANLAGEN);
    	}
    }
    __NotInitialized = false;
  }

}
