/**
 * Copyright (c) 2004 Logos GmbH Rostock (www.logos-ib.de).
 * All rights reserved.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL  LOGOS GMBH  OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package de.inovat.kex.bast.common.objectwrapper;


/*
 * Revision 1.4  2008/09/01 08:17:22  Schawaller
 * neue Methode zum Sortieren der FS
 */
import java.util.Arrays;
import java.util.List;
import java.util.WeakHashMap;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.ConfigurationObjectType;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.debug.Debug;
import de.inovat.kex.bast.common.*;

/**
 * kapselt Instancen des DatenModellTypes fahrStreifen.
 *
 * <p>
 * <b>Achtung:</b>
 * <br>
 * Das Attribute "Typ" wird zur Sortierung der Fahrstreifen benutzt.
 * Es wird davon ausgegangen, dass die  Zust&auml;nde/Werte des
 * Attributtypes att.fahrStreifenTyp von aussen nach innen aufsteigen.
 * 
 * @author LOGOS GmbH Rostock (www.logos-ib.de)
 * @author Ralf Trier
 * @version $Id: Fahrstreifen.java 442 2008-11-10 06:59:32Z Schawaller $
 */
public class Fahrstreifen
    extends ConfigurationObjectWrapper {

  /** DebugLogger fuer Debug-Ausgaben. */
  private static Debug debug = Debug.getLogger();

  private static WeakHashMap<String,Fahrstreifen> __Cache = new WeakHashMap<String,Fahrstreifen>();
  private static ConfigurationObjectType __ConfigurationObjectType;

  private static ConfigurationObjectType __ConfigurationObjectTypeFahrstreifenQuelleASCII;
  private static ConfigurationObjectType __ConfigurationObjectTypeFahrstreifenQuelleVirtuell;
  private static ConfigurationObjectType __ConfigurationObjectTypeDeLve;
  private static ConfigurationObjectType __ConfigurationObjectTypeDeAxl;

  private static AttributeGroup __AtgFahrStreifen;

  private String _Typ;
//  private int _TypNr;

  private String _Lage;
  private int _LageNr;

  private String _ErsatzFahrstreifenPID;
  private FahrstreifenQuelle _FahrstreifenQuelle;
  private DEKanal.Axl _DEKanalAxl;

  private MessQuerschnitt _MessQuerschnitt;


  /**
   * <p>setzt den MessQuerschnitt, zu dem der fahrstreifen geh&ouml;rt.</p>
   *
   * @param messQuerschnitt MessQuerschnitt
   */
  protected void setMessQuerschnitt(MessQuerschnitt messQuerschnitt) {
    this._MessQuerschnitt = messQuerschnitt;
  }

  /**
   * <p>gibt den MessQuerschnitt zur&uuml;ck, zu dem der Fahrstreifen
   * geh&ouml;rt, wenn dieser irgenwann gesetzt worden ist.</p>
   *
   * @return MessQuerschnitt
   */
  public MessQuerschnitt getMessQuerschnitt() {
    return _MessQuerschnitt;
  }

  /**
   * gibt ein Object auf die FahrstreifenQuelle zurück, das eine Instance der
   * Klassen FahrstreifenQuelleASCII, FahrstreifenQuelleDeLve oder
   * FahrstreifenQuelleVirtuell ist.
   *
   * @return FahrstreifenQuelle
   */
  public FahrstreifenQuelle getFahrstreifenQuelle() {
    return _FahrstreifenQuelle;
  }

  /**
   * gibt ein Object auf den DEKanal.Axl zurück, wenn f&uuml;r den Fahrstreifen
   * Achslasten erfa&szlig;t werden..
   *
   * @return DEKanal f&uuml;r Achslastdatenerfassung oder null, wenn keine
   *   Erfassung.
   */
  public DEKanal.Axl getDEKanalAxl() {
    if (AxlLastMessStelle.isNotInitialized()) {
      try {
        AxlLastMessStelle.getAll();
      }
      catch (Exception e) {
        debug.error("Fehler beim Laden der AchslastMessStellen", e);
      }
    }
    return _DEKanalAxl;
  }

  /**
   * setzt den DEKanal.Axl.
   * 
   * @param deKanal ToDo.
   */
  public void setDEKanalAxl(DEKanal.Axl deKanal) {
    _DEKanalAxl = deKanal;
  }

  /**
   * gibt true zur&uuml;ck, wenn die Quelle vom DEKanal.Axl ist.
   *
   * @return boolean
   * @throws DataModellMismatchException Exception.
   */
  public boolean isAchslastErfassung() throws DataModellMismatchException {
    if (AxlLastMessStelle.isNotInitialized()) {
      try {
        AxlLastMessStelle.getAll();
      }
      catch (Exception e) {
        debug.error("Fehler beim Laden der AchslastMessStellen", e);
      }
    }
    return (_DEKanalAxl != null);
  }

  /**
   * gibt den ErsatzFahrstreifen zurück oder null, wenn keiner definiert wurde.
   *
   * @throws DataModellObjectNotFoundException Exception.
   * @throws DataModellMismatchException Exception.
   * @return Fahrstreifen
   */
  public Fahrstreifen getErsatzFahrstreifen()
      throws DataModellObjectNotFoundException, DataModellMismatchException {

    if (_ErsatzFahrstreifenPID != null) {
      return getInstance(_ErsatzFahrstreifenPID);
    }
    else {
      return null;
    }
  }

  /**
   * gibt den Typ des Fahrstreifen zurück: SonstigerFahrStreifen,
   * Kriechspur, HFS, 1ÜFS, 2ÜFS usw..
   *
   * @return String
   */
  public String getTyp() {
    return _Typ;
  }


  private Fahrstreifen(String pid)
      throws DataModellObjectNotFoundException, DataModellMismatchException {
    super(pid);
  }

  private Fahrstreifen(ConfigurationObject confObj) throws DataModellMismatchException {
    super(confObj);
  }

  protected void initData(ConfigurationObject fahrstreifen) throws DataModellMismatchException {

    Data oData = fahrstreifen.getConfigurationData(__AtgFahrStreifen);
    if (oData != null) {

      if (oData.getUnscaledValue("Typ").isState()) {
        _Typ = oData.getUnscaledValue("Typ").getState().getName();
//        _TypNr = oData.getUnscaledValue("Typ").intValue();
      }

      if (oData.getUnscaledValue("Lage").isState()) {
        _Lage = oData.getUnscaledValue("Lage").getState().getName();
      }
      else {
        _Lage = oData.getUnscaledValue("Lage").getValueText();
      }
      _LageNr = oData.getUnscaledValue("Lage").intValue();


      if (oData.getReferenceValue("ErsatzFahrStreifen").getSystemObject() != null) {
        _ErsatzFahrstreifenPID = oData.getReferenceValue("ErsatzFahrStreifen").getSystemObject().getPid();
      }

        SystemObject obj = oData.getReferenceValue("FahrStreifenQuelle").getSystemObject();
        if (obj != null) {
          if (obj instanceof ConfigurationObject) {
            _FahrstreifenQuelle = (FahrstreifenQuelle) Fahrstreifen.getFahrstreifenquelle( (ConfigurationObject)obj);
            if (_FahrstreifenQuelle instanceof DEKanal.Lve) {
              ((DEKanal.Lve)_FahrstreifenQuelle).setFahrstreifen(this);
            }
            if (_FahrstreifenQuelle instanceof DEKanal.Axl) {
              ((DEKanal.Axl)_FahrstreifenQuelle).setFahrstreifen(this);
            }
          }
          else {
            throw new DataModellMismatchException("Die ObjektReferenz FahrstreifenQuelle des Fahrstreifen \"" + getPID() +
                                                  "\" verweist nicht auf ein ConfigurationsObject", getPID());
          }
        }
        else {
//          throw new DataModellMismatchException("Fahrstreifen \"" + getPID() +
//                                                "\" keine ObjektReferenz auf eine FahrstreifenQuelle", getPID());
        }
    }
    __Cache.put(this.getPID(), this);
  }

  /**
   * gibt die Instance der FahrStreifen mit der PID zurück. Es wird eine
   * DataModellObjectNotFoundException geworfen, wenn die PID nicht vorhanden
   * ist oder das zur PID gehörende Objekt nich vom typ fahrStreifen ist. Eine
   * DataModellMismatchException wird geworfen, wenn Objektreferenzen auf
   * falsche Typen verweisen bzw. diese nicht gesetzt sind.
   *
   * @param pid String
   * @throws DataModellMismatchException Exception.
   * @return Fahrstreifen
   */
  public static Fahrstreifen getInstance(String pid) throws DataModellMismatchException {
    Fahrstreifen retValue = __Cache.get(pid);
    if (retValue == null) {
      Object o = NetzClientDavConnection.getInstance().getDataModel().getObject(pid);
      if ((o != null) && (o instanceof ConfigurationObject)) {
        retValue = Fahrstreifen.getInstanceIntern( (ConfigurationObject)o );
      }
    }
    return retValue;
  }

  protected static Fahrstreifen getInstance(ConfigurationObject fahrStreifen) throws DataModellMismatchException {
    Fahrstreifen retValue = __Cache.get(fahrStreifen.getPid());
    if (retValue == null) {
      retValue = Fahrstreifen.getInstanceIntern(fahrStreifen);
    }
    return retValue;
  }

  private static synchronized Fahrstreifen getInstanceIntern(ConfigurationObject fahrStreifen) throws DataModellMismatchException {
    Fahrstreifen retValue = __Cache.get(fahrStreifen.getPid());
    if (retValue == null) {
      if (__ConfigurationObjectType == null) {
        Fahrstreifen.init();
      }
      retValue = new Fahrstreifen(fahrStreifen);
    }
    return retValue;
  }


  /**
   * gibt die DatenModellObjekte vom typ fahrstreifen zurück, wenn keiner dieser
   * Objekte Fehler in den ObjektReferenzen aufweist.
   * DataModellMismatchException wird geworfen, wenn Objektreferenzen auf
   * falsche Typen verweisen bzw. diese nicht gesetzt sind.
   *
   * @throws DataModellMismatchException Exception.
   * @return Fahrstreifen[]
   */
  public static Fahrstreifen[] getAll() throws DataModellMismatchException {
    if (__ConfigurationObjectType == null) {
      Fahrstreifen.init();
    }
    Fahrstreifen[] retValue;
    if (__ConfigurationObjectType!=null) {
      List<SystemObject> list = __ConfigurationObjectType.getElements();
      retValue = new Fahrstreifen[list.size()];
      for (int i = 0; i < list.size(); i++) {
        retValue[i] = Fahrstreifen.getInstanceIntern((ConfigurationObject) list.get(i));
      }
    }
    else {
      retValue = new Fahrstreifen[0];
    }
    return retValue;
  }

  public String toString() {
    StringBuffer sb = new StringBuffer();
    sb.append("Fahrstreifen - PID: \"").append(getPID()).append("\"  ");
    sb.append("Name: \"").append(getName());
    return sb.toString();
  }

  public String getMultilineRepresentation(int indent) {
    String lf = System.getProperty("line.separator");
    StringBuffer sIndent = new StringBuffer();
    for (int i=0; i<indent; i++) {
      sIndent.append(' ');
    }
    StringBuffer sb = new StringBuffer();
    sb.append(sIndent).append("PID: \"").append(getPID()).append("\"").append(lf);
    sb.append(sIndent).append("Name: \"").append(getName()).append("\"").append(lf);
    sb.append(sIndent).append("Typ: \"").append(_Typ).append("\"").append(lf);
    sb.append(sIndent).append("ErsatzFahrstreifenPID: \"").append(_ErsatzFahrstreifenPID).append("\"").append(lf);
    if (_FahrstreifenQuelle != null) {
      sb.append(sIndent).append("FahrstreifenQuelle: \"").append(_FahrstreifenQuelle).append("\"");
    }
    else {
      sb.append(sIndent).append("FahrstreifenQuelle ist null");
    }
    return sb.toString();
  }

  public String getMultilineRepresentation() {
   return getMultilineRepresentation(0);
  }


  ConfigurationObjectType getType() {
    return __ConfigurationObjectType;
  }

  /**
   * Gibt die Lage als Nummer wieder.
   *
   * @return int Lage als Nummer
   */
  public int getLageNr(){
    return _LageNr;
  }

  public String getLage() {
    return _Lage;
  }

  public String getTreeString() {
    String s;
    if (_LageNr < 0) {
      s = _Typ + " " + String.valueOf(Math.abs(_LageNr)) + ". FS rechts vom HFS";
    }
    else {
      s = _Lage;
    }
    if (_FahrstreifenQuelle==null) {
      return s + " (ohne Datenerfassung)";
    }
    else {
      return s;
    }
  }


  private static FahrstreifenQuelle getFahrstreifenquelle(ConfigurationObject confObj)
      throws DataModellMismatchException  {

    if (__ConfigurationObjectTypeDeLve==null) {
      DataModel model = NetzClientDavConnection.getInstance().getDataModel();
      __ConfigurationObjectTypeDeLve = (ConfigurationObjectType)model.getType("typ.deLve");
      __ConfigurationObjectTypeDeAxl = (ConfigurationObjectType)model.getType("typ.deAxl");
      __ConfigurationObjectTypeFahrstreifenQuelleASCII = (ConfigurationObjectType)model.getType("typ.verkehrsDatenQuelleASCII");
      __ConfigurationObjectTypeFahrstreifenQuelleVirtuell = (ConfigurationObjectType)model.getType("typ.verkehrsDatenQuelleVirtuell");
    }

    if (confObj!=null) {
      if (confObj.isOfType(__ConfigurationObjectTypeDeLve)) {
        return (FahrstreifenQuelle) DEKanal.getInstance(confObj);
      }
      if (confObj.isOfType(__ConfigurationObjectTypeFahrstreifenQuelleASCII)) {
        return new FahrstreifenQuelleASCII(confObj);
      }
      if (confObj.isOfType(__ConfigurationObjectTypeFahrstreifenQuelleVirtuell)) {
        return new FahrstreifenQuelleVirtuell(confObj);
      }
      if (confObj.isOfType(__ConfigurationObjectTypeDeAxl)) {
        return (FahrstreifenQuelle) DEKanal.getInstance(confObj);
      }
    }
    return null;
  }


  public static final void sortByFsLage(Fahrstreifen[] objectArray) {
    Arrays.sort(objectArray, new FSComperatorVonAussenNachInnen());
  }


  static class FSComperatorVonAussenNachInnen implements java.util.Comparator<Fahrstreifen> {
    FSComperatorVonAussenNachInnen() {
    }

    public boolean equals(Object obj) {
      return (this == obj);
    }

    public int compare(Fahrstreifen fs01, Fahrstreifen fs02) {
      int retValue = 0;
      if (fs01 != fs02) {
        retValue = (fs01._LageNr - fs02._LageNr);
      }
      return retValue;
    }
  }


  private static synchronized void init() {
    if (__ConfigurationObjectType==null) {
      DataModel model = NetzClientDavConnection.getInstance().getDataModel();
      __ConfigurationObjectType = (ConfigurationObjectType)model.getType("typ.fahrStreifen");
      __AtgFahrStreifen = model.getAttributeGroup("atg.fahrStreifen");
      List<SystemObject> list = __ConfigurationObjectType.getElements();
      model.getConfigurationData(list, __AtgFahrStreifen);
    }
  }

}



