/**
 * Copyright (c) 2004 Logos GmbH Rostock (www.logos-ib.de).
 * All rights reserved.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL  LOGOS GMBH  OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package de.inovat.kex.bast.common.objectwrapper;

/*
 * Revision 1.5  2008/09/01 08:17:27  Schawaller
 * zugriiffsmethoden, javadoc, sortierung
 */
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Comparator;
import java.util.Date;
import java.util.HashMap;
import java.util.List;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.ConfigurationObjectType;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.debug.Debug;
import de.inovat.kex.bast.common.DataModellMismatchException;
import de.inovat.kex.bast.common.NetzClientDavConnection;

/**
 * kapselt den DatenModellTyp langZeitZählstelle
 *
 * @author LOGOS GmbH Rostock (www.logos-ib.de)
 * @author Ralf Trier
 * @version $Id: LangZeitZaehlStelle.java 442 2008-11-10 06:59:32Z Schawaller $
 */
public class LangZeitZaehlStelle extends ConfigurationObjectWrapper {

  /** DebugLogger fuer Debug-Ausgaben. */
  private static Debug debug = Debug.getLogger();

  private static HashMap<String,LangZeitZaehlStelle>  __CacheByPid = new HashMap<String,LangZeitZaehlStelle>();
  private static HashMap<Integer,LangZeitZaehlStelle> __CacheByBastNr = new HashMap<Integer,LangZeitZaehlStelle>();

  private static LangZeitZaehlStelle[] __LZZArray;

  private static ConfigurationObjectType __ConfigurationObjectType;
  private static AttributeGroup __AtgLangZeitZaehlstelle;

  private int _LzzNummer;
  private String _LzzName;
  private String _TelefonNummer;
  private Bundesland _Bundesland;
  private StrassenKlasse _StrassenKlasse;
  private String _LzzArt;
  private String _PidStrasse;
  private int _VonNK;
  private int _NachNK;
  private double _Station;
  private long _Beginn;
  private long _Ende;
  private String _KlassifizierungsgruppeBAST;

  private int _InNK;
  private String _InNahziel;
  private String _InFernziel;
  private String _InStr;
  private AuswerteQuerschnitt _InAuswertequerschnitt;

  private int _GegenNK;
  private String _GegenNahziel;
  private String _GegenFernziel;
  private String _GegenStr;
  private AuswerteQuerschnitt _GegenAuswertequerschnitt;

  private boolean _IsFunkStation;

  private AuswerteQuerschnitt[] _AuswerteQuerschnitt;


  private LangZeitZaehlStelle(ConfigurationObject confObj) throws DataModellMismatchException {
    super(confObj);
  }


  /**
   * Interne statitsiche Methode zum Erzeugen der Objekte.
   *
   * @param confObj ConfigurationObject
   * @return LangZeitZaehlStelle
   * @throws DataModellMismatchException Exception.
   */
  private static synchronized LangZeitZaehlStelle getInstanceIntern(ConfigurationObject confObj) throws DataModellMismatchException {
    LangZeitZaehlStelle retValue = __CacheByPid.get(confObj.getPid());
    if (retValue == null) {
      if (__ConfigurationObjectType == null) {
        LangZeitZaehlStelle.init();
      }
      retValue = new LangZeitZaehlStelle(confObj);
    }
    return retValue;
  }


  /**
   * Gibt die Instance der LangZeitZaehlStelle mit der PID zurück. Es wird eine
   * DataModellObjectNotFoundException geworfen, wenn die PID nicht vorhanden
   * ist oder das zur PID gehörende Objekt nich vom typ langZeitZählstelle ist.
   * Eine DataModellMismatchException wird geworfen, wenn Objektreferenzen auf
   * falsche Typen verweisen bzw. diese nicht gesetzt sind.
   *
   * @param pid String
   * @throws DataModellMismatchException Exception.
   * @return LangZeitZaehlStelle
   */
  public static LangZeitZaehlStelle getInstance(String pid) throws DataModellMismatchException {
    LangZeitZaehlStelle retValue = __CacheByPid.get(pid);
    if (retValue == null) {
      Object o = NetzClientDavConnection.getInstance().getDataModel().getObject(pid);
      if ((o!=null) && (o instanceof ConfigurationObject)) {
        retValue = LangZeitZaehlStelle.getInstanceIntern((ConfigurationObject)o);
      }
    }
    return retValue;
  }



  /**
   * Gibt die Instance der LangZeitZaehlStelle mit der angegebenen Bast-Nummer zurück. Es wird eine
   * DataModellObjectNotFoundException geworfen, wenn die PID nicht vorhanden
   * ist oder das zur PID gehörende Objekt nich vom typ langZeitZählstelle ist.
   * Eine DataModellMismatchException wird geworfen, wenn Objektreferenzen auf
   * falsche Typen verweisen bzw. diese nicht gesetzt sind.
   *
   * @param bastNr int
   * @throws DataModellMismatchException Exception.
   * @return LangZeitZaehlStelle
   */
  public static LangZeitZaehlStelle getInstance(int bastNr) throws DataModellMismatchException {
    LangZeitZaehlStelle retValue = __CacheByBastNr.get(bastNr);
    if (retValue == null) {
      LangZeitZaehlStelle.getAll();
      retValue = __CacheByBastNr.get(bastNr);
    }
    return retValue;
  }


  /**
   * gibt die DatenModellObjekte vom typ langZeitZählstelle zurück, wenn keiner
   * dieser Objekte Fehler in den ObjektReferenzen aufweist.
   * DataModellMismatchException wird geworfen, wenn Objektreferenzen auf
   * falsche Typen verweisen bzw. diese nicht gesetzt sind.
   *
   * @throws DataModellMismatchException Exception.
   * @return LangZeitZaehlStelle[]
   */
  public static LangZeitZaehlStelle[] getAll() throws DataModellMismatchException {
    if (__LZZArray == null) {
      if (__ConfigurationObjectType == null) {
        LangZeitZaehlStelle.init();
      }
      if (__ConfigurationObjectType != null) {
        DataModel model = NetzClientDavConnection.getInstance().getDataModel();
        List<SystemObject> list = model.getType("typ.langZeitZählstelle").getElements();
        __LZZArray = new LangZeitZaehlStelle[list.size()];

		  // ***------------------------------------------
	      // BW-DAUZ: Probleme beim initLZZ, da MQ werden teilweise durch virtuelleMQ ersetzt
	      // ---> try-catch-Block für Fehler_Abfang. Die fehlerhaften LZZ werden aus dem Array entfernt.
	      int mitFehler=0;
        for (int i = 0; i < list.size(); i++) {

			// ***------------------------------------------
	        try{
		        __LZZArray[i-mitFehler] = LangZeitZaehlStelle.getInstanceIntern((ConfigurationObject) list.get(i));
	        }
	        catch (Exception e){
		        mitFehler++;
		        debug.error("Fehler bei Initialisierung der Langzeitzählstelle", e);
	        }
        }

	      // ***------------------------------------------
	      __LZZArray=Arrays.copyOfRange(__LZZArray, 0, list.size()-mitFehler);
      }
      else {
        __LZZArray = new LangZeitZaehlStelle[0];
      }
    }
    return __LZZArray.clone();
  }


  /**
   * gibt die LangzeitZ&auml;hlstellenWrapperObjekte zur&uuml;ck, die der Verkehrmengenerfassung dienen und
   * die keine Fehler in den ObjektReferenzen aufweisen.
   * DataModellMismatchException wird geworfen, wenn Objektreferenzen auf
   * falsche Typen verweisen bzw. diese nicht gesetzt sind.
   *
   *
   * @return LangZeitZaehlStelle[]
   * @throws DataModellMismatchException DataModellMismatchException.
   */
  public static LangZeitZaehlStelle[] getAllLve() throws DataModellMismatchException {

    LangZeitZaehlStelle[] lzzArray = getAll();

    ArrayList<LangZeitZaehlStelle> list = new ArrayList<LangZeitZaehlStelle>(lzzArray.length);

    for (int i = 0; i < lzzArray.length; i++) {
      if (!lzzArray[i].isAchslastErfassung()) {
        list.add(lzzArray[i]);
      }
    }
    return list.toArray(new LangZeitZaehlStelle[list.size()]).clone();
  }

  /**
   * gibt die LangzeitZ&auml;hlstellenWrapperObjekte zur&uuml;ck, die der Achlastdatenfassung dienen und
   * die keine Fehler in den ObjektReferenzen aufweisen.
   * DataModellMismatchException wird geworfen, wenn Objektreferenzen auf
   * falsche Typen verweisen bzw. diese nicht gesetzt sind.
   *
   * @throws DataModellMismatchException Exception.
   * @return LangZeitZaehlStelle[]
   */
  public static LangZeitZaehlStelle[] getAllAxl() throws DataModellMismatchException {

    LangZeitZaehlStelle[] lzzArray = getAll();

    ArrayList<LangZeitZaehlStelle> list = new ArrayList<LangZeitZaehlStelle>(2);

    for (int i = 0; i < lzzArray.length; i++) {
      if (lzzArray[i].isAchslastErfassung()) {
        list.add(lzzArray[i]);
      }
    }
    return list.toArray(new LangZeitZaehlStelle[list.size()]).clone();
  }


  protected void initData(ConfigurationObject confObj) throws DataModellMismatchException {

    Data oData = confObj.getConfigurationData(__AtgLangZeitZaehlstelle);

    _LzzName = oData.getTextValue("ZählstellenName").getText();
    _LzzNummer = oData.getUnscaledValue("ZählstellenNummer").intValue();

    _TelefonNummer = oData.getTextValue("TelefonNummer").getText();

    if (oData.getUnscaledValue("LandesNummer").isState()) {
      _Bundesland = Bundesland.getInstanceByCode(oData.getUnscaledValue("LandesNummer").intValue());
    }

    if (oData.getUnscaledValue("StraßenKlasse").isState()) {
      try {
        _StrassenKlasse = StrassenKlasse.getInstanceByName(oData.getUnscaledValue("StraßenKlasse").getState().getName());
      }
      catch (Exception e) {
        debug.error("Fehler bei der Zuordnung der Strassenklasse", e);
      }
    }
    else {
	    debug.error(String.format("%s StrassenKlasse ist kein Status", this.getPID()));
    }

    _PidStrasse = oData.getReferenceValue("Straße").getSystemObject().getPid();

    if (oData.getUnscaledValue("vonNK").isNumber()) {
      _VonNK = oData.getUnscaledValue("vonNK").intValue();
    }
    if (oData.getUnscaledValue("nachNK").isNumber()) {
      _NachNK = oData.getUnscaledValue("nachNK").intValue();
    }
    if (oData.getUnscaledValue("Station").isNumber()) {
      _Station = oData.getUnscaledValue("Station").doubleValue();
    }
    try {
      _Beginn = oData.getTimeValue("Beginn").getMillis();
    }
    catch (Exception e) {
      e.printStackTrace();
    }
    try {
      _Ende = oData.getTimeValue("Ende").getMillis();
    }
    catch (Exception e) {
      e.printStackTrace();
    }

    if (oData.getUnscaledValue("KlassifizierungsgruppeBAST").isState()) {
      _KlassifizierungsgruppeBAST = oData.getUnscaledValue("KlassifizierungsgruppeBAST").getState().getName();
    }

    Data atlData = oData.getItem("InRichtung");
    if (atlData.getUnscaledValue("NK").isNumber()) {
      _InNK = atlData.getUnscaledValue("NK").intValue();
    }
    _InNahziel = atlData.getTextValue("Nahziel").getText();
    _InFernziel = atlData.getTextValue("Fernziel").getText();
    _InStr = atlData.getTextValue("Bezeichnung").getText();
    _InAuswertequerschnitt = AuswerteQuerschnitt.getInstance((ConfigurationObject)atlData.getReferenceValue("Auswertequerschnitt").getSystemObject());
    _InAuswertequerschnitt.setLangZeitZaehlStelle(this);

    atlData = oData.getItem("GegenRichtung");
    if (atlData.getUnscaledValue("NK").isNumber()) {
      _GegenNK = atlData.getUnscaledValue("NK").intValue();
    }
    _GegenNahziel = atlData.getTextValue("Nahziel").getText();
    _GegenFernziel = atlData.getTextValue("Fernziel").getText();
    _GegenStr = atlData.getTextValue("Bezeichnung").getText();
    _GegenAuswertequerschnitt = AuswerteQuerschnitt.getInstance((ConfigurationObject)atlData.getReferenceValue("Auswertequerschnitt").getSystemObject());
    _GegenAuswertequerschnitt.setLangZeitZaehlStelle(this);
      //try-catch da NullPointerException durch confObj.getObjectSet(..) DR  inovat 21.07.2010
      List<SystemObject> listMQ = new ArrayList<SystemObject>();
      try {
         listMQ = confObj.getObjectSet("AuswerteQuerschnitte").getElements();
      } catch(Exception e){

      }
    _AuswerteQuerschnitt = new AuswerteQuerschnitt[listMQ.size()];
    for (int mqNr = 0; (mqNr < listMQ.size()); mqNr++) {
      _AuswerteQuerschnitt[mqNr] = AuswerteQuerschnitt.getInstance((ConfigurationObject) listMQ.get(mqNr));
    }

    // bestimmen ob an Funk oder nicht
    if (_InAuswertequerschnitt!=null) {
      if (_InAuswertequerschnitt.getMessQuerschnitt() !=null) {
        Fahrstreifen[] fsArray = _InAuswertequerschnitt.getMessQuerschnitt().getFahrstreifen();
        if ((fsArray!=null) && (fsArray.length>0) && (fsArray[0]!=null)) {
          FahrstreifenQuelle quelle = fsArray[0].getFahrstreifenQuelle();
          if ((quelle!=null) && (quelle instanceof DEKanal.Lve)) {
            EAK eak = ((DEKanal.Lve)quelle).getEAK();
            if (eak!=null) {
              SteuerModul sm = eak.getSteuerModul();
              if (sm!=null) {
                KriBus bus = sm.getBus();
                if (bus!=null) {
                  _IsFunkStation = bus.isFunkBus();
                }
                else {
	                debug.info(String.format("kein Bus für SteuerModul %s", sm.getPID()));
                }
              }
              else {
	              debug.info(String.format("kein SteuerModul für EAK %s", eak.getPID()));
              }
            }
            else {
	            debug.info(String.format("kein EAK für Quelle %s", quelle));
            }
          }
        }
      }

    }
    __CacheByPid.put(this.getPID(), this);
    __CacheByBastNr.put(this.getBastNummer(), this);
  }







  public double getStation() {
    return _Station;
  }

  public int getBastNummer() {
	  try {
		  return Integer.valueOf(_TelefonNummer);
	  }
	  catch (Exception e){
		  return (_LzzNummer % 10000);
	  }
  }

  /**
   * gibt die vierstellige Bastnummer als String zur&uuml;ck.
   *
   * @return String
   */
  public String getBastNummerAsString() {
    int iBastNummer = getBastNummer();
    String sBastNummer = String.valueOf(iBastNummer);
    while (sBastNummer.length() < 4) {
      sBastNummer = '0' + sBastNummer;
    }
    return sBastNummer;
  }

  /**
   * gibt die achtstellige Langzeitz&auml;stellenNummer (TK25 + BASt-Nummer) als String zur&uuml;ck.
   *
   * @return String
   */
  public String getLzzNummerAsString() {
    String sLzzNummer = String.valueOf(_LzzNummer);
    while (sLzzNummer.length() < 8) {
      sLzzNummer = '0' + sLzzNummer;
    }
    return sLzzNummer;
  }

  /**
   * gibt die vierstellige TK-Blatt-Nr als String zur&uuml;ck.
   *
   * @return String
   */
  public String getTK25AsString() {
    int iTK25 = (int)(_LzzNummer / 10000);
    String sTK25 = String.valueOf(iTK25);
    while (sTK25.length() < 4) {
      sTK25 = '0' + sTK25;
    }
    return sTK25;
  }



  public AuswerteQuerschnitt getInAuswertequerschnitt() {
    return _InAuswertequerschnitt;
  }

  public StrassenKlasse getStrassenKlasse() {
    return _StrassenKlasse;
  }

  public int getVonNK() {
    return _VonNK;
  }

  public String getKlassifizierungsgruppeBAST() {
    return _KlassifizierungsgruppeBAST;
  }

  public int getAnzahlFahrzeugklassen() {
    if (_KlassifizierungsgruppeBAST.startsWith("8")) {
      return 9;
    }
    if (_KlassifizierungsgruppeBAST.startsWith("5")) {
      return 6;
    }
    if (_KlassifizierungsgruppeBAST.startsWith("2")) {
      return 2;
    }
    return 2;
  }

  public String getPidStrasse() {
    return _PidStrasse;
  }

  public Strasse getStrasse() {
    try {
      return Strasse.getInstance(_PidStrasse);
    }
    catch (DataModellMismatchException ex) {
      ex.printStackTrace();
      return null;
    }
    catch (Exception ex) {
      ex.printStackTrace();
      return null;
    }
  }

  public int getInNK() {
    return _InNK;
  }

  public String getGegenStr() {
    return _GegenStr;
  }

  public long getEnde() {
    return _Ende;
  }

  public String getGegenNahziel() {
    return _GegenNahziel;
  }

  public int getGegenNK() {
    return _GegenNK;
  }

  public String getInNahziel() {
    return _InNahziel;
  }

  public String getInFernziel() {
    return _InFernziel;
  }

  public Bundesland getBundesland() {
    return _Bundesland;
  }

  public String getGegenFernziel() {
    return _GegenFernziel;
  }

  public AuswerteQuerschnitt getGegenAuswertequerschnitt() {
    return _GegenAuswertequerschnitt;
  }

  public int getNachNK() {
    return _NachNK;
  }

  public long getBeginn() {
    return _Beginn;
  }

  public String getInStr() {
    return _InStr;
  }

  public String getLzzArt() {
    return _LzzArt;
  }





  /**
   * gibt die Nummer der Zählstelle zuück
   *
   * @return int
   */
  public int getLzzNummer() {
    return _LzzNummer;
  }
  /**
   * gibt den Namen der Zählstelle zuück
   *
   * @return int
   */
  public String getLzzName() {
    return _LzzName;
  }


  public boolean isFunkstation() {
    return _IsFunkStation;
  }


  /**
   * Gibt die AuswerteQuerschnitte wieder, die <b>nicht</b> IN- oder GEGEN-
   * AuswerteQuerschnitte sind.
   *
   * @return AuswerteQuerschnitt[]
   */
  public AuswerteQuerschnitt[] getAuswerteQuerschnitte() {
    return _AuswerteQuerschnitt.clone();
  }


//  /**
//   * Sortiert die angegebenen LangZeitZaehlStellen nach den Namen.
//   *
//   * @param lzzArray LangZeitZaehlStelle[]
//   */
//  public static void sortByName(LangZeitZaehlStelle[] lzzArray) {
//    Comparator<LangZeitZaehlStelle> comparator = new Comparator<LangZeitZaehlStelle>() {
//      public boolean equals(Object o) {
//        return (o == this);
//      }
//      public int compare(LangZeitZaehlStelle lzz01, LangZeitZaehlStelle lzz02) {
//        String s1 = lzz01.getName();
//        String s2 = lzz02.getName();
//        if ((s1!=null) && (s2!=null)) {
//          return (s1.compareTo(s2));
//        }
//        if (s1 == null) {
//          return 1;
//        }
//        return -1;
//      }
//    };
//    Arrays.sort(lzzArray, comparator);
//  }



  public String toString() {
    StringBuffer sb = new StringBuffer();
    sb.append(_LzzNummer).append(" - ").append(_LzzName);
    return sb.toString();
  }


  public String getMultilineRepresentation(int indent) {
    String lf = System.getProperty("line.separator");
    StringBuffer sIndent = new StringBuffer();
    for (int i=0; i<indent; i++) {
      sIndent.append(' ');
    }
    StringBuffer sb = new StringBuffer();

    sb.append(sIndent).append("PID: \"").append(getPID()).append("\"").append(lf);
    sb.append(sIndent).append("Name: \"").append(getName()).append("\"").append(lf);

    sb.append(sIndent).append("ZählstellenNummer: \"").append(_LzzNummer).append("\"").append(lf);
    sb.append(sIndent).append("ZählstellenName: \"").append(_LzzName).append("\"").append(lf);

    sb.append(sIndent).append("Land: \"").append(_Bundesland).append("\"").append(lf);
    sb.append(sIndent).append("Strassenklasse: \"").append(_StrassenKlasse).append("\"").append(lf);
    sb.append(sIndent).append("Zählstellenart: \"").append(_LzzArt).append("\"").append(lf);
    sb.append(sIndent).append("Strasse: \"").append(_PidStrasse).append("\"").append(lf);
    sb.append(sIndent).append("VonNK: \"").append(_VonNK).append("\"").append(lf);
    sb.append(sIndent).append("NachNK: \"").append(_NachNK).append("\"").append(lf);
    sb.append(sIndent).append("Station: \"").append(_Station).append("\"").append(lf);
    if (_Beginn == 0) {
      sb.append(sIndent).append("Beginn: \"nicht bekannt\"").append(lf);
    }
    else {
      sb.append(sIndent).append("Beginn: \"").append(new Date(_Beginn)).append("\"").append(lf);
    }
    if (_Ende == 0) {
      sb.append(sIndent).append("Ende: \"-\"").append(lf);
    }
    else {
      sb.append(sIndent).append("Ende: \"").append(new Date(_Ende)).append("\"").append(lf);
    }
    sb.append(sIndent).append("KlassifizierungsgruppeBAST: \"").append(_KlassifizierungsgruppeBAST).append("\"").append(lf);

    sb.append(sIndent).append("InRichtung:").append(lf);
    sb.append(sIndent).append("  NK: \"").append(_InNK).append("\"").append(lf);
    sb.append(sIndent).append("  Nahziel: \"").append(_InNahziel).append("\"").append(lf);
    sb.append(sIndent).append("  Fernziel: \"").append(_InFernziel).append("\"").append(lf);
    sb.append(sIndent).append("  Str: \"").append(_InStr).append("\"").append(lf);
    sb.append(sIndent).append("  Auswertequerschnitt:").append(lf);
    sb.append(_InAuswertequerschnitt.getMultilineRepresentation(indent + 8));

    sb.append(sIndent).append("GegenRichtung:").append(lf);
    sb.append(sIndent).append("  NK: \"").append(_GegenNK).append("\"").append(lf);
    sb.append(sIndent).append("  Nahziel: \"").append(_GegenNahziel).append("\"").append(lf);
    sb.append(sIndent).append("  Fernziel: \"").append(_GegenFernziel).append("\"").append(lf);
    sb.append(sIndent).append("  Str: \"").append(_GegenStr).append("\"").append(lf);
    sb.append(sIndent).append("  Auswertequerschnitt:").append(lf);
    sb.append(_GegenAuswertequerschnitt.getMultilineRepresentation(indent + 8));


    sb.append(sIndent).append("Messquerschnitte:").append(lf);
    if ( (_AuswerteQuerschnitt != null) && (_AuswerteQuerschnitt.length > 0)) {
      for (int i = 0; i < _AuswerteQuerschnitt.length; i++) {
        sb.append(sIndent).append("  Messquerschnitt " + i + ": " + _AuswerteQuerschnitt[i].getMultilineRepresentation(indent + 8));
      }
    }
    else {
      sb.append(sIndent).append("  keine Messquerschnitte vorhanden");
    }
    return sb.toString();
  }

  /**
   * getMultilineRepresentation
   *
   * @return String
   */
  public String getMultilineRepresentation() {
    return getMultilineRepresentation(0);
  }

  public String getTreeString() {
    return this.getName();
  }

  public String getToolTipText() {
    try {
      return this.getStrasse().getTreeString() + ' ' + this.getInFernziel() + " - " + this.getGegenFernziel();
    }
    catch (Exception e) {
      return null;
    }
  }

  ConfigurationObjectType getType() {
    return __ConfigurationObjectType;
  }

  /**
   * gibt die Fahrstreifen der Z&auml;hlstelle in der Reihenfolge InRichtg.HFS,
   * InRichtg.&Uuml;FS01, InRichtg.&Uuml;FS02 .... GegenRichtg.&Uuml;FS02,
   * GegenRichtg.&Uuml;FS01, GegenRichtg.HFS zur&uuml;ck.
   *
   * @return Fahrstreifen[]
   */
  public Fahrstreifen[] getFahrstreifen() {
    Fahrstreifen[] fsInArray = null;
    Fahrstreifen[] fsGegenArray = null;
    AuswerteQuerschnitt inAuswertequerschnitt = _InAuswertequerschnitt;
    if (inAuswertequerschnitt != null) {
      MessQuerschnitt mq = inAuswertequerschnitt.getMessQuerschnitt();
      if (mq != null) {
        fsInArray = mq.getFahrstreifen();
      }
    }
    AuswerteQuerschnitt gegenAuswertequerschnitt = _GegenAuswertequerschnitt;
    if (gegenAuswertequerschnitt != null) {
      MessQuerschnitt mq = gegenAuswertequerschnitt.getMessQuerschnitt();
      if (mq != null) {
        fsGegenArray = mq.getFahrstreifen();
      }
    }

    // die Fahrstreifen werden von rechts (HauptFS) nach links (ÜFS01, ÜFS02..) zurückgegeben.
    // die Ausgabe in das BASt-Format erfolgt aber in der Reihenfolge
    // HFS 1.RFB, ÜFS01 1.RFB, ÜFS02 1.RFB, ... , ÜFS02 2.RFB, ÜFS01 2.RFB, HFS 2.RFB
    ArrayList<Fahrstreifen> fsList = new ArrayList<Fahrstreifen>();
    if (fsInArray != null) {
      for (int i = 0; i < fsInArray.length; i++) {
        fsList.add(fsInArray[i]);
      }
    }
    if (fsGegenArray != null) {
      for (int i = (fsGegenArray.length-1); i >=0; i--) {
        fsList.add(fsGegenArray[i]);
      }
    }
    return fsList.toArray(new Fahrstreifen[fsList.size()]);
  }

  /**
   * gibt true zur&uuml;ck, wenn die Zählstelle Achslastdaten erfasst.
   *
   * @return boolean
   * @throws DataModellMismatchException Exception.
   */
  public boolean isAchslastErfassung() throws DataModellMismatchException {
    boolean bReturnValue = false;
    Fahrstreifen[] fsArray = getFahrstreifen();
    for (int i=0; ((i<fsArray.length) && (!bReturnValue)); i++) {
      bReturnValue = fsArray[i].isAchslastErfassung();
    }
    return bReturnValue;
  }


  /** Vergleichoperator für Sortierung der Lzz nach der BastNr */
  private static Comparator<LangZeitZaehlStelle> __SorterByBastNr =
      new Comparator<LangZeitZaehlStelle>()
  {
    public boolean equals(Object obj) {
      return (obj==this);
    }
    public int compare(LangZeitZaehlStelle lzz01, LangZeitZaehlStelle lzz02) {
      return lzz01.getBastNummer() - lzz02.getBastNummer();
    }
  };

  /**
   * <p>Sortiert das Array von ConfigurationObjectWrapper Objekten nach Namen.</p>
   * @param lzzArray ToDo.
   */
  public static final void sortByBastNr(LangZeitZaehlStelle[] lzzArray) {
    Arrays.sort(lzzArray, __SorterByBastNr);
  }


  /**
   * Initialisiert die statischen Datenmodeltypen.
   */
  private static synchronized void init() {
    if (__ConfigurationObjectType==null) {
      DataModel model = NetzClientDavConnection.getInstance().getDataModel();
      __ConfigurationObjectType = (ConfigurationObjectType)model.getType("typ.langZeitZählstelle");
      __AtgLangZeitZaehlstelle = model.getAttributeGroup("atg.langZeitZählstelle");
      List<SystemObject> list = __ConfigurationObjectType.getObjects();
      model.getConfigurationData(list, __AtgLangZeitZaehlstelle);
    }
  }

}
