/*
 * Copyright 2008 by inovat, Dipl.-Ing. H. C. Kniß
 * ALL RIGHTS RESERVED.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL inovat OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */



package de.inovat.kex.bast.exporter;

//~ NICHT JDK IMPORTE =========================================================

import de.inovat.kex.bast.SendeBAStStatusDaten;
import de.inovat.kex.bast.common.ChannelException;

import java.util.List;

//~ JDK IMPORTE ===============================================================

//~ INTERFACES ================================================================

/**
 * Interface stellt die Schnittstelle für die Server-Kommunikation beim BASt-Band-Export
 * dar (s. {@link de.inovat.kex.bast.exporter.FileDataChannel#flush()}).
 *
 * @author inovat, innovative systeme - verkehr - tunnel - technik
 * @author Liliya Givorgizova (LG)
 * @version $Revision$ / $Date$ / ($Author$)
 */
public interface IServerExport {

    /**
     * Exportiere die Datei nach Server.
     *
     * @param klasseFileDataChannel Klasse FileDataChannel, um die Methode
     * {@link FileDataChannel#writeHeaderFTP(java.io.BufferedOutputStream)} zugängig zu machen.
     * @param lineBuffer Puffer für die Daten.
     * @param bastStatusDaten Instanz von SendeBAStStatusDaten für die Meldungen.
     * @param dateiName Dateiname.
     * @param unterOrdner Unterordner (s. {@link de.inovat.kex.bast.exporter.FileDataChannel#_unterordnerName}).
     *
     * @return true - wenn der Export erfolgreich war, false - sonst.
     *
     * @throws ChannelException Exception.
     */
    public boolean exportiereDateiNachServer(FileDataChannel klasseFileDataChannel, List<Object> lineBuffer, SendeBAStStatusDaten bastStatusDaten, String dateiName, String unterOrdner) throws ChannelException;

    /**
     * Trenne die Verbindung mit dem Server.
     *
     * @return Exception, falls einen _Fehler aufgetretten ist
     *  oder null - wenn die Verbindung zum Server erfolgreich getrennt wird.
     */
    public Exception trenneVerbindungMitServer();
}


//~Formatiert mit 'inovat Kodierkonvention' am 25.06.13
