/*
 * Copyright (c) 2010 by inovat, innovative systeme - verkehr - tunnel - technik, Dipl.-Ing. H. C. Kniss
 *
 * This file is part of de.inovat.kex.bast.importer.ErzeugeTestDatensatzBAStV2004
 *
 * de.inovat.kex.bast.importer.ErzeugeTestDatensatzBAStV2004 is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * de.inovat.kex.bast.importer.ErzeugeTestDatensatzBAStV2004 is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * <p>
 * You should have received a copy of the GNU General Public License
 * along with de.inovat.kex.bast.importer.ErzeugeTestDatensatzBAStV2004; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * Contact Information:
 * inovat, Dipl.-Ing. H. C. Kniss
 * Koelner Strasse 30
 * D-50859 Köln
 * +49 (0)2234 4301 800
 * info@invat.de
 * www.inovat.de
 */



package de.inovat.kex.bast.importer;

//~ NICHT JDK IMPORTE =========================================================

import de.bsvrz.sys.funclib.application.StandardApplicationRunner;
import de.bsvrz.sys.funclib.commandLineArgs.ArgumentList;
import de.inovat.kex.bast.KonstantenBast.BAST_FORMAT;
import de.inovat.kex.bast.common.ChannelException;
import de.inovat.kex.bast.common.DataModellMismatchException;
import de.inovat.kex.bast.common.objectwrapper.*;
import org.apache.commons.net.ftp.FTPClient;

import java.io.*;
import java.util.*;

//~ JDK IMPORTE ===============================================================

//~ KLASSEN ===================================================================

/**
 * Die Klasse dient als Hilfsklasse zum Erzeugen von Testdatensätzen.
 * Mittels dieser Klasse können Testdaten für das BastBestandsbandformat (8Plus1, 5Plus1, 2, 1)
 * sowie VV2007 (8Plus1, 5Plus1, 2, 1) erzeugt werden.
 * Die Daten werden in die entsprechenden Verzeichnisse das FTP-Servers abgelegt.
 *
 * @author inovat, innovative systeme - verkehr - tunnel - technik
 * @author Dagmar Rostek (DR)
 * @version $Revision$ / $Date$ / ($Author$)
 */
public class ErzeugeTestDatensatzBASt {

    /** Header Anzahl der Fahrzeugarten. */
    private static String ANZAHL_FAHRZEUGARTEN[] = { "01", "02", "06", "09" };

    /** Header Anzahl der Fahrzeuggruppen, 01 für Datensätze vom Typ 1, sonst immer 02. */
    private static String ANZAHL_FAHRZEUGGRUPPEN[] = { "00", "01", "02" };

    /** Header eines Datensatzes vom Typ 1 besteht aus der Gesamtmenge der Kraftfahrzeuge Kfz */
    private static String HEADER_1 = "KFZ";

    /** Header eines Datensatzes vom Typ 2. */
    private static String HEADER_2 = "KFZ Lkw";

    /** Header eines Datensatzes vom Typ 5 Plus 1. */
    private static String HEADER_5PLUS1 = "PLZ PmA Bus LoA Lzg Son";

    /** Headerdaten 8 Plus 1 Fahrzeugarten. */
    private static String HEADER_8PLUS1 = "Mot Pkw LfW Pma Bus LoA LmA Sat Son";

    /** Headerdaten  Fahrzeuggruppen 5Plus1 und 8Plus1. */
    private static String HEADER_FG = "KFZ SV";

    /** Leerzeichen. */
    private static String LEERZEICHEN = " ";

    //~ FELDER ================================================================

    private String            _host       = "192.168.2.102";
    private String            _passwd     = "inovat";
    private String            _user       = "SWE_BAST";
    private ArrayList<Object> _lineBuffer = new ArrayList<Object>();

    /** FTPClient für die FTP Connetion. */
    private FTPClient _ftpClient = new FTPClient();
    private int       _anzahlLZZ;
    private String    _endDate;

    /** Filename. */
    private StringBuilder _fileName;

    /** Header 1 entsprechend des zu erzeugenden Datensatzes. */
    private StringBuffer _header1;

    /** Header 2 entsprechend des zu erzeugenden Datensatzes. */
    private StringBuffer _header2;

    /** Header 3 entspreched des zu erzeugenden Datensatzes. */
    private StringBuffer _header3;
    private BAST_FORMAT _bastFormat;

    /**
     * Integer Attribut, dass die Konvertierungsroutine angibt, folgende Werte sind zugelassen:
     * 1, 2, 5, 8. Wenn kein Wert als Parameter übergeben wurde, werden Datensätze für alle
     * Konvertierungen durchgeführt.
     */
    private int    _iFZKlassifizierung;
    private int    _jahr;
    private int    _monat;
    private String _startDate;

    //~ KONSTRUKTOREN  (und vom Konstruktor verwendete Methoden) ==============

    /**
     * Konstruktor.
     *
     * @param host   Host.
     * @param user   FTP-Server user.
     * @param passwd FTP-Server passwort.
     * @param bastFormat  BASt-Band format (=2004, 2007)
     * @param iKonvertierung Konvertierung (1, 2, 5Plus1, 8Plus1)
     * @param dStartDate   Startdatum.
     * @param dEndDate    Enddatum.
     * @param anzahlLZZ  Anzahl der Langzeitzählstellen für Testdatensätze.
     */
    public ErzeugeTestDatensatzBASt(String host, String user, String passwd, BAST_FORMAT bastFormat, int iKonvertierung, String dStartDate, String dEndDate, int anzahlLZZ) {
        _host               = host;
        _user               = user;
        _passwd             = passwd;
        _iFZKlassifizierung = iKonvertierung;
        _startDate          = dStartDate;
        _endDate            = dEndDate;
        _bastFormat = bastFormat;
        _anzahlLZZ          = anzahlLZZ;

        try {
            erzeugeTestDaten();
        }
        catch (ChannelException e) {
            e.printStackTrace();

            throw new RuntimeException(e);
        }
        catch (DataModellMismatchException e) {
            e.printStackTrace();

            throw new RuntimeException(e);
        }
    }

    //~ METHODEN ==============================================================

    private void aktualisiereVerbindung() {
        _ftpClient = verbindeMitFTPServer();
        loginToFTPServer();
        changeExportDirektoryFTPServer();
    }

    /**
     * Die Methode bestimmt den ersten Headerdatensatz für die übergebene Langzeitzählstelle.
     * @param lzz Langzeitzählstelle für die der Testdatensatz erzeugt wird.
     */
    private void bestimmeHeader1(LangZeitZaehlStelle lzz) {
        String sBundesLandNummer = String.valueOf(lzz.getBundesland().getKennung());

        while (sBundesLandNummer.length() == 1) {
            sBundesLandNummer = '0' + sBundesLandNummer;
        }

        _header1 = new StringBuffer("H");
        _header1.append(lzz.getLzzNummerAsString()).append(' ');
        _header1.append(sBundesLandNummer).append(' ');
        _header1.append(lzz.getStrassenKlasse().getAbbreviation()).append(' ');

        Strasse strasse = lzz.getStrasse();

        try {
            _header1.append(strasse.getNummer()).append(strasse.getZusatz());
        }
        catch (Exception ex1) {
            ex1.printStackTrace();
        }

        while (_header1.length() < 21) {
            _header1.append(' ');
        }

        String sLzzName = lzz.getLzzName();

        if (sLzzName.length() > 25) {
            sLzzName.substring(0, 24);
        }

        _header1.append(lzz.getLzzName());

        while (_header1.length() < 46) {
            _header1.append(' ');
        }

        _header1.append("V2.0;");
        _lineBuffer.add(_header1);
    }

    /**
     * Die Methode bestimmt den zweiten Headerdatensatz für die übergebene Langzeitzählstelle.
     * @param lzz Langzeitzählstelle für die der Testdatensatz erzeugt wird.
     */
    private void bestimmeHeader2(LangZeitZaehlStelle lzz) {
        AuswerteQuerschnitt inAuswertequerschnitt    = lzz.getInAuswertequerschnitt();
        int                 fsCountInRichtung        = inAuswertequerschnitt.getMessQuerschnitt().getFahrstreifen().length;
        String              inFernziel               = lzz.getInFernziel();
        String              sInHimmelsrichtung       = inAuswertequerschnitt.getHimmelsrichtung();
        AuswerteQuerschnitt gegenAuswertequerschnitt = lzz.getInAuswertequerschnitt();
        int                 fsCountGegenRichtung     = gegenAuswertequerschnitt.getMessQuerschnitt().getFahrstreifen().length;
        String              gegenFernziel            = lzz.getGegenFernziel();
        String              sGegenHimmelsrichtung    = gegenAuswertequerschnitt.getHimmelsrichtung();

        _header2 = new StringBuffer("R");

        if (fsCountInRichtung < 10) {
            _header2.append('0');
        }

        _header2.append(String.valueOf(fsCountInRichtung)).append(' ');

        if (fsCountGegenRichtung < 10) {
            _header2.append('0');
        }

        _header2.append(String.valueOf(fsCountGegenRichtung)).append(' ');

        if (inFernziel.length() > 20) {
            inFernziel = inFernziel.substring(0, 19);
        }

        _header2.append(inFernziel);

        while (_header2.length() < 27) {
            _header2.append(' ');
        }

        _header2.append(sInHimmelsrichtung).append(' ');

        if (gegenFernziel.length() > 20) {
            gegenFernziel = gegenFernziel.substring(0, 19);
        }

        _header2.append(gegenFernziel);

        while (_header2.length() < 49) {
            _header2.append(' ');
        }

        _header2.append(sGegenHimmelsrichtung).append(';');
        _lineBuffer.add(_header2);
    }

    /**
     * Die Methode besimmt ausgehend von dem übergebenem Wert den entsprechenden Headerdatensatz.
     * <li> 1 -> Headerdatensatz 1. </li>
     * <li> 2 -> Headerdatensatz 2. </li>
     * <li> 5 -> Headerdatensatz 5Plus1. </li>
     * <li> 8 -> Headerdatensatz 8Plus1. </li>
     * Andere Zahlenwerte führen zu einer Infomeldung und werden nicht ausgewertet.
     *
     */
    private void bestimmeHeader3() {
        _header3 = new StringBuffer("S");

        if (_iFZKlassifizierung == 1) {
            _header3.append(ANZAHL_FAHRZEUGARTEN[0]);
            _header3.append(LEERZEICHEN);
            _header3.append(ANZAHL_FAHRZEUGGRUPPEN[0]);
            _header3.append(LEERZEICHEN);
            _header3.append(HEADER_1);
        } else if (_iFZKlassifizierung == 2) {
            _header3.append(ANZAHL_FAHRZEUGARTEN[1]);
            _header3.append(LEERZEICHEN);
            _header3.append(ANZAHL_FAHRZEUGGRUPPEN[0]);
            _header3.append(LEERZEICHEN);
            _header3.append(HEADER_2);
        } else if (_iFZKlassifizierung == 3) {
            _header3.append(ANZAHL_FAHRZEUGARTEN[1]);
            _header3.append(LEERZEICHEN);
            _header3.append(ANZAHL_FAHRZEUGGRUPPEN[2]);
            _header3.append(LEERZEICHEN);
            _header3.append(HEADER_FG);
            _header3.append(LEERZEICHEN);
            _header3.append(HEADER_5PLUS1);
        } else if (_iFZKlassifizierung == 5) {
            _header3.append(ANZAHL_FAHRZEUGARTEN[1]);
            _header3.append(LEERZEICHEN);
            _header3.append(ANZAHL_FAHRZEUGGRUPPEN[3]);
            _header3.append(LEERZEICHEN);
            _header3.append(HEADER_FG);
            _header3.append(LEERZEICHEN);
            _header3.append(HEADER_8PLUS1);
        } else if (_iFZKlassifizierung == 0) {}
        else {
            System.out.println("Es konnte kein Testdatensatz erzeugt werden, da die übergebene Nummer keinen Datensatz repräsentiert. Zulässige werte sind 1, 2, 5 und 8!");
        }

        _header3.append(";");
        _lineBuffer.add(_header3);
    }

    /**
     * Die Methode wechselt in das Exportverzeichnis für den gestarteten Export.
     *
     *         @return true, wenn der Wechsel in das Exportverzeichnis auf dem FTP-Server ohne
     *         Exception geklappt hat, false sonst.
     */
    public boolean changeExportDirektoryFTPServer() {
        boolean changeDirectoryOK;

        try {
            _ftpClient.changeWorkingDirectory("Import\\TestDaten");
            changeDirectoryOK = true;
        }
        catch (IOException e) {
            e.printStackTrace();

            throw new RuntimeException(e);
        }

        return changeDirectoryOK;
    }

    /**
     * Die Methode erzeugt den dummydatensatz.
     * @param lzz Langzeitzählstelle für die der Datensatz erzeugt wird.
     */
    private void erzeugeDatenSatz1(LangZeitZaehlStelle lzz) {
        AuswerteQuerschnitt inAuswertequerschnitt    = lzz.getInAuswertequerschnitt();
        int                 fsCountInRichtung        = inAuswertequerschnitt.getMessQuerschnitt().getFahrstreifen().length;
        AuswerteQuerschnitt gegenAuswertequerschnitt = lzz.getInAuswertequerschnitt();
        int                 fsCountGegenRichtung     = gegenAuswertequerschnitt.getMessQuerschnitt().getFahrstreifen().length;
        Calendar            cal                      = Calendar.getInstance();

        _jahr  = Integer.parseInt(_startDate.substring(0, 2));
        _monat = Integer.parseInt(_startDate.substring(2, 4));
        cal.set(_jahr, _monat);

        Random randomGenerator = new Random();

        for (int tag = 1; tag < cal.getMaximum(Calendar.DAY_OF_MONTH); tag++) {
            for (int stunde = 1; stunde < 24; stunde++) {
                StringBuffer stundendatensatz = new StringBuffer();

                stundendatensatz.append(_jahr);

                if (_monat < 10) {
                    stundendatensatz.append(0);
                }

                stundendatensatz.append(_monat);

                if (tag < 10) {
                    stundendatensatz.append(0);
                }

                stundendatensatz.append(tag);
                stundendatensatz.append(LEERZEICHEN);

                if (stunde < 10) {
                    stundendatensatz.append(0);
                }

                stundendatensatz.append(stunde);
                stundendatensatz.append(":00");
                stundendatensatz.append(LEERZEICHEN);

                for (int anzahlFSRI = 1; anzahlFSRI < fsCountInRichtung; anzahlFSRI++) {

                    // Zufallswert der Mengen der KFZ.
                    stundendatensatz.append(randomGenerator.nextInt(100));
                    stundendatensatz.append("-");
                    stundendatensatz.append(LEERZEICHEN);
                }

                for (int anzahlFSRII = 1; anzahlFSRII < fsCountGegenRichtung; anzahlFSRII++) {
                    stundendatensatz.append(randomGenerator.nextInt(100));
                    stundendatensatz.append("-");
                    stundendatensatz.append(LEERZEICHEN);
                }

                _lineBuffer.add(stundendatensatz);
            }
        }
    }

    /**
     * Die Methode erzeugt den dummydatensatz.
     */
    private void erzeugeDatenSatz2(LangZeitZaehlStelle lzz) {

        // To change body of created methods use File | Settings | File Templates.
    }

    private void erzeugeDatenSatz5Plus1(LangZeitZaehlStelle lzz) {

        // To change body of created methods use File | Settings | File Templates.
    }

    /**
     * Die Methode erzeugt einen dummydatensatz.
     */
    private void erzeugeDatenSatz8Plus1(LangZeitZaehlStelle lzz) {

        // To change body of created methods use File | Settings | File Templates.
    }

    /**
     * Die Methode bestimmt den Filenamen für die Ausgabe der Datei auf den FTP-Server.
     * Aufbau des Dateinamens:
     * <ul>
     * <li> Bundesland:Kürzel (Position 1-2) --- <b> Bsp: NW</b></li>
     * <li> Zählstellennummer (Position 3-6) --- <b> Bsp: 4606 </b></li>
     * <li> Version (optional) (Position 7-8) ---<b> Bsp: _1 </b></li>
     * <li> Suffix Jahr(yy) MonatsCode (Position 1-3) --- <b> Bsp: 974 </b></li>
     * </ul>
     * @param zaehlStelle Zählstelle für den der Dateiname konstruiert wird.
     * @param year   Jahr.
     * @param month  Monat für den der BASt-Export gestartet wird.
     */
    protected void erzeugeFilenameV2004(LangZeitZaehlStelle zaehlStelle, int month, int year) {
        _fileName = new StringBuilder();
        _fileName.append(zaehlStelle.getBundesland().getAbbreviation());
        _fileName.append(zaehlStelle.getBastNummerAsString());
        _fileName.append(".");
        _fileName.append(year);

        if (month <= 9) {
            _fileName.append(month);
        } else {
            switch (month) {
            case 10 :
                _fileName.append("a");

                break;

            case 11 :
                _fileName.append("b");

                break;

            case 12 :
                _fileName.append("c");

                break;
            }
        }
    }

    /**
     * Die Methode erzeugt anzahlDerZuErzeugendenTestdatensätze Testdatensätze für jede Konfiguration, der einer BASt-Importdatei der
     * Verkehrsmengendaten Version 2004 entspricht.
     * @throws ChannelException Exception.
     * @throws DataModellMismatchException  Exception.
     */
    public void erzeugeTestDaten() throws ChannelException, DataModellMismatchException {
        LangZeitZaehlStelle[] alleLZZ = new LangZeitZaehlStelle[LangZeitZaehlStelle.getAll().length];

        try {
            alleLZZ = LangZeitZaehlStelle.getAll();
        }
        catch (DataModellMismatchException e) {
            e.printStackTrace();
        }

        aktualisiereVerbindung();

        for (int i = 0; i < _anzahlLZZ; i++) {
            _lineBuffer.clear();
            bestimmeHeader1(alleLZZ[i]);
            bestimmeHeader2(alleLZZ[i]);
            bestimmeHeader3();

            if (_iFZKlassifizierung == 8) {
                erzeugeDatenSatz8Plus1(alleLZZ[i]);
            } else if (_iFZKlassifizierung == 5) {
                erzeugeDatenSatz5Plus1(alleLZZ[i]);
            } else if (_iFZKlassifizierung == 2) {
                erzeugeDatenSatz2(alleLZZ[i]);
            } else if (_iFZKlassifizierung == 1) {
                erzeugeDatenSatz1(alleLZZ[i]);
            }

            schreibeDatenAufFTPServer(alleLZZ[i]);
        }

        schliesseVerbindungFTPServer();
        System.exit(0);
    }

    /**
     * Die Methode prüft ob die übergebene Datei auf dem FTP-Server existiert.
     * @param fileName   Filename der bzgl. Existenz überprüft wird.
     * @return true, wenn die Datei existiert, false sonst.
     */
    public boolean fileExistiert(String fileName) {
        boolean fileExistiert = false;
        String  alleFiles[];

        if (_ftpClient != null) {
            try {
                alleFiles = _ftpClient.listNames();

                if (alleFiles != null) {
                    for (String file : alleFiles) {
                        if (file.equals(fileName)) {
                            fileExistiert = true;
                        }
                    }
                } else {
                    schliesseVerbindungFTPServer();
                    aktualisiereVerbindung();
                }
            }
            catch (IOException e) {
                e.printStackTrace();

                throw new RuntimeException(e);
            }
        } else {
            aktualisiereVerbindung();
            System.out.println("aktualisierung der Verbindung durchgeführt.");
        }

        return fileExistiert;
    }

    //~ GET METHODEN ==========================================================

    private static int getKonvertierungsFormatAsInt(String konvertierung) {
        if ("8PLUS1".equalsIgnoreCase(konvertierung)) {
            return 8;
        }

        if ("5PLUS1".equalsIgnoreCase(konvertierung)) {
            return 5;
        }

        if ("2".equalsIgnoreCase(konvertierung)) {
            return 2;
        }

        if ("1".equalsIgnoreCase(konvertierung)) {
            return 1;
        }

        return 0;
    }

    //~ METHODEN ==============================================================

    private boolean loginToFTPServer() {
        boolean loginOK;

        try {
            _ftpClient.login(_user, _passwd);
            loginOK = true;
        }
        catch (IOException e) {
            e.printStackTrace();

            throw new RuntimeException(e);
        }

        return loginOK;
    }

    /**
     * Main ....
     * @param args keine die ausgewertet werden.
     * @throws ChannelException Exception.
     */
    public static void main(String[] args) throws ChannelException {

	    //TODO ???
        System.setErr(System.out);

        ArgumentList argumentList = new ArgumentList(args);

        if (argumentList.hasArgument("-BAStImportTestDatenErzeugung")) {
            if (!argumentList.hasArgument("-host")) {
                System.out.println("Parameter \"-host\" muss angegeben werden.");
                printVerwendung();
                System.exit(0);
            }

            String host = argumentList.fetchArgument("-host").asString();

            if (!argumentList.hasArgument("-user")) {
                System.out.println("Parameter \"-user\" muss angegeben werden.");
                printVerwendung();
                System.exit(0);
            }

            String user = argumentList.fetchArgument("-user").asString();

            if (!argumentList.hasArgument("-passwd")) {
                System.out.println("Parameter \"-passwd\" muss angegeben werden.");
                printVerwendung();
                System.exit(0);
            }

            String passwd = argumentList.fetchArgument("-passwd").asString();

            if (!argumentList.hasArgument("-bastformat")) {
                System.out.println("Parameter \"-bastformat\" muss angegeben werden.");
                printVerwendung();
                System.exit(0);
            }

            String                     sBAStFormat = argumentList.fetchArgument("-bastformat").asString();
            BAST_FORMAT bastFormat  = BAST_FORMAT.ermittleFormat(sBAStFormat);

            // Ungültiges Format:
            if ((bastFormat == null) || (bastFormat == BAST_FORMAT.AXL_EF1)) {
                System.out.println("Parameter \"-bastformat\" muss einer der Werte sein: V2004, V2007.");
                printVerwendung();
                System.exit(0);
            }

            int iKonvertierung;

            if (argumentList.hasArgument("-konvertierung")) {
                iKonvertierung = getKonvertierungsFormatAsInt(argumentList.fetchArgument("-konvertierung").asString());
            } else {
                iKonvertierung = 0;
            }

            String sStartDate;

            if (!argumentList.hasArgument("-von")) {
                try {
                    sStartDate = (new BufferedReader(new InputStreamReader(System.in))).readLine().trim();
                }
                catch (IOException e) {
                    e.printStackTrace();

                    throw new RuntimeException(e);
                }
            } else {
                sStartDate = argumentList.fetchArgument("-von").asString();
            }

            String sEndDate;

            if (!argumentList.hasArgument("-bis")) {
                try {
                    sEndDate = (new BufferedReader(new InputStreamReader(System.in))).readLine();
                }
                catch (IOException e) {
                    e.printStackTrace();

                    throw new RuntimeException(e);
                }
            } else {
                sEndDate = argumentList.fetchArgument("-bis").asString();
            }

            int    anzahlLZZ = 1;
            String lzz       = argumentList.fetchArgument("-anzahlLZZ").asString();

            if (!lzz.equals("")) {
                anzahlLZZ = Integer.parseInt(argumentList.fetchArgument("-anzahlLZZ").asString());
            }

            BAStImportSteuerung bastImport = new BAStImportSteuerung();

            StandardApplicationRunner.run(bastImport, args);
            bastImport.anmeldenDavEmpfaengerBAStImport();
            new ErzeugeTestDatensatzBASt(host, user, passwd, bastFormat, iKonvertierung, sStartDate, sEndDate, anzahlLZZ);
        } else {
            System.out.println("Das Programm wird beendet, da keine Parameter für die Testdatenerzeugung angegeben worden sind.");
            printVerwendung();
            System.exit(0);
        }
    }

    public static void printVerwendung() {
        System.out.println("Neben den Parametern für den Datenverteiler sind anzugeben: ");
        System.out.println("    -host=192.168.1.102 (IP-Adresse des FTP-Servers, auf den die Testdaten abgelegt werden.)");
        System.out.println("    -user=SWE_BAST (Username für die Login Information auf den FTP-Server.)");
        System.out.println("    -passwd=inovat (Passwort für den Login auf den FTP-Server)");
        System.out.println("    -bastformat=V2004|V2007");
        System.out.println("    -konvertierung=8PLUS1|5PLUS1|2|1 (Optionale Parameter Konvertierungsroutine, wenn keiner angegeben wird, werden für alle Konvertierungen Testdaten erzeugt.)");
        System.out.println("    -dir=directoryZumSpeichern");
        System.out.println("und");
        System.out.println("    -von=\"JJ.MM.TT\" ");
        System.out.println("    -bis=\"JJ.MM.TT\" ");
        System.out.println("    -anzahlLZZ=int (Die Anzahl der Zählstellen für die eine Testdatei erzeugt werden soll.)");
    }

    /**
     * Die Methode schließt die Verbindung zum FTP-Server.
     */
    private void schliesseVerbindungFTPServer() {
        try {
            _ftpClient.disconnect();
        }
        catch (IOException e) {
            e.printStackTrace();

            throw new RuntimeException(e);
        }

        // To change body of created methods use File | Settings | File Templates.
    }

    /**
     * Die Methode schreibt die ermittelten Datensätze auf den FTP-Server.
     * @throws ChannelException  Excpetion.
     */
    private void schreibeDateiAufFTPServer() throws ChannelException {
        BufferedOutputStream bufferedAusgabe = null;

        System.out.println("fileName ... " + _fileName.toString() + " lineBuffer " + _lineBuffer.size());

        if (!fileExistiert(_fileName.toString())) {
            try {
                bufferedAusgabe = new BufferedOutputStream(_ftpClient.appendFileStream(_fileName.toString()));
            }
            catch (IOException e) {}

            try {
                while ((_lineBuffer.size() > 0)) {
                    assert bufferedAusgabe != null;
                    bufferedAusgabe.write(_lineBuffer.remove(0).toString().getBytes());
                    bufferedAusgabe.write(System.getProperty("line.separator").getBytes());
                }

                assert bufferedAusgabe != null;
                bufferedAusgabe.close();
            }
            catch (IOException e) {
                throw new ChannelException("Channel ist schon geschlossen!", e);
            }
        }
    }

    /**
     * Die Methode schreibt die Daten auf den FTP-Server.
     * @param lzz   Langzeitzählstelle.
     * @throws ChannelException    Exception.
     */
    private void schreibeDatenAufFTPServer(LangZeitZaehlStelle lzz) throws ChannelException {
        erzeugeFilenameV2004(lzz, _monat, _jahr);
        schreibeDateiAufFTPServer();
    }

    /**
     * Die Methode stellt eine Verbindung zum FTP-Server her.
     * @return  FTPClient.
     */
    private FTPClient verbindeMitFTPServer() {
        _ftpClient = new FTPClient();

        try {
            _ftpClient.connect(_host);
        }
        catch (IOException e) {
            e.printStackTrace();

            return null;
        }

        return _ftpClient;
    }
}


//~Formatiert mit 'inovat Kodierkonvention' am 31.07.12
