/*
 * Copyright (c) 2016 by inovat, innovative systeme - verkehr - tunnel - technik, Dipl.-Ing. H. C. Kniss
 *
 * This file is part of de.inovat.kex.bast.importer.ImportLineSplitterBASt
 *
 * de.inovat.kex.bast.importer.ImportLineSplitterBASt is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  de.inovat.kex.bast.importer.ImportLineSplitterBASt is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with de.inovat.kex.bast.importer.ImportLineSplitterBASt.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  Diese Datei ist Teil von de.inovat.kex.bast.importer.ImportLineSplitterBASt.
 *
 *  de.inovat.kex.bast.importer.ImportLineSplitterBASt ist Freie Software: Sie können es unter den Bedingungen
 *  der GNU General Public License, wie von der Free Software Foundation,
 *  Version 3 der Lizenz oder (nach Ihrer Wahl) jeder späteren
 *  veröffentlichten Version, weiterverbreiten und/oder modifizieren.
 *
 *  de.inovat.kex.bast.importer.ImportLineSplitterBASt wird in der Hoffnung, dass es nützlich sein wird, aber
 *  OHNE JEDE GEWÄHRLEISTUNG, bereitgestellt; sogar ohne die implizite
 *  Gewährleistung der MARKTFÄHIGKEIT oder EIGNUNG FÜR EINEN BESTIMMTEN ZWECK.
 *  Siehe die GNU General Public License für weitere Details.
 *
 *  Sie sollten eine Kopie der GNU General Public License zusammen mit diesem
 *  Programm erhalten haben. Wenn nicht, siehe <http://www.gnu.org/licenses/>.
 *
 * Contact Information:
 * inovat, Dipl.-Ing. H. C. Kniss
 * An der Krautwiese 37
 * D-53783 Eitorf
 * +49 (0)2243 8464 193
 * info@inovat.de
 * www.inovat.de
 */

package de.inovat.kex.bast.importer;

//~ NICHT JDK IMPORTE =========================================================

import de.bsvrz.dav.daf.main.SendSubscriptionNotConfirmed;
import de.bsvrz.sys.funclib.debug.Debug;
import de.inovat.kex.bast.SendeBAStStatusDaten;
import de.inovat.kex.bast.common.DataModellMismatchException;
import de.inovat.kex.bast.common.objectwrapper.Fahrstreifen;
import de.inovat.kex.bast.common.objectwrapper.LangZeitZaehlStelle;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;

//~ JDK IMPORTE ===============================================================

//~ KLASSEN ===================================================================

/**
 * Hilfsklasse um aus den Zeilen der BASt-Band VerkehrsMenge2004 Importdatei die Werte auszulesen.
 *
 * @author inovat, innovative systeme - verkehr - tunnel - technik
 * @author Dagmar Rostek (DR)
 * @version $Revision$ / $Date$ / ($Author$)
 */
public class ImportLineSplitterBASt {

    /** Konstante repräsentiert die Fahrzeuggruppe Busse und Pkw mit Anhänger. */
    private static final String BPA = "BPA";

    /** Konstante  reprästentiert die Fahrzeuggruppe Leichtverkehr ohne Anhänger. */
    private static final String LVO = "LVo";

    /** DebugLogger fuer Debug-Ausgaben. */
    private static Debug debug = Debug.getLogger();

    /** Konstante repräsentiert die Fahrzeuggruppe schwerer Güterverkehr. */
    private static final String SGV = "SGV";

    /**
     * Attribut ist nicht relevant und wird nicht ausgelesen (ist für Fahrzeuggruppe SV Schwerverkehr
     * in der Klassifizierungen 5 Plus 1 und 8 Plus 1 eingefügt).
     */
    private static final String ATTRIBUT_UNDEFINIERT = "X";

    /**
     * Array mit allen relevanten Werten (DaV-Attributnamen) für einen Datensatz BASt-Format V2004, Klassifizierung 2.
     * Klassifizierung 2 besteht aus 2 Fahrzeuggruppen und 0 Fahrzeugarten.
     * Array enthält Fahrzeugarten.
     */
    private static final String[] AR_VERKEHRSDATEN_LZZ_2_V2004_ARTEN = {};

    /**
     * Array mit allen relevanten Werten (DaV-Attributnamen) für einen Datensatz BASt-Format V2004, Klassifizierung 2.
     * Klassifizierung 2 besteht aus 2 Fahrzeuggruppen und 0 Fahrzeugarten.
     * Array enthält Fahrzeuggruppen.
     */
    private static final String[] AR_VERKEHRSDATEN_LZZ_2_V2004_GRUPPEN = { "qKfz", "qLkwÄ" };

    /**
     *
     *  Array mit allen relevanten Werten (DaV-Attributnamen) für einen Datensatz BASt-Format V2004, Klassifizierung 5 Plus 1.
     *  Klassifizierung 5 Plus 1 besteht aus 2 Fahrzeuggruppen und 6 Fahrzeugarten.
     *  Array enthält Fahrzeugarten.
     */
    private static final String[] AR_VERKEHRSDATEN_LZZ_5_PLUS_1_V2004_ARTEN = { "qPkwG", "qPkwA", "qBus", "qLkw", "qLkwK", "qKfzNk" };

    /**
     * Array mit allen relevanten Werten (DaV-Attributnamen) für einen Datensatz BASt-Format V2004, Klassifizierung 5 Plus 1.
     * Klassifizierung 5 Plus 1 besteht aus 2 Fahrzeuggruppen und 6 Fahrzeugarten.
     * Array enthält Fahrzeuggruppen.
     */
    private static final String[] AR_VERKEHRSDATEN_LZZ_5_PLUS_1_V2004_GRUPPEN = { "qKfz", ATTRIBUT_UNDEFINIERT };

    /**
     * Array mit allen relevanten Werten (DaV-Attributnamen) für einen Datensatz BASt-Format V2004, Klassifizierung 8 Plus 1.
     *     Klassifizierung 8 Plus 1 besteht aus 2 Fahrzeuggruppen und 9 Fahrzeugarten.
     *     Array enthält Fahrzeugarten.
     */
    private static final String[] AR_VERKEHRSDATEN_LZZ_8_PLUS_1_V2004_ARTEN = { "qKrad", "qPkw", "qLfw", "qPkwA", "qBus", "qLkw", "qLkwA", "qSattelKfz", "qKfzNk" };

    /**
     * Array mit allen relevanten Werten (DaV-Attributnamen) für einen Datensatz BASt-Format V2004, Klassifizierung 8 Plus 1.
     *     Klassifizierung 8 Plus 1 besteht aus 2 Fahrzeuggruppen und 9 Fahrzeugarten.
     *     Array enthält Fahrzeuggruppen.
     */
    private static final String[] AR_VERKEHRSDATEN_LZZ_8_PLUS_1_V2004_GRUPPEN = { "qKfz", ATTRIBUT_UNDEFINIERT };

    //~ FELDER ================================================================

    private int[] _bPAValues = { 0, 40, 50, 60, 70, 80, 90, 100, 110, 120 };

    // private Map<String, HashMap<Integer, Integer>> _gKlasseVerkehrsMengeGKlasse = new HashMap<String, HashMap<Integer, Integer>>();

    /** Geschwindigkeitsklassen. */
    private int[]    _lVoValues                        = { 0, 40, 50, 60, 70, 80, 90, 100, 110, 120, 130, 140, 150, 160, 170, 180 };
    private int[]    _sGVValues                        = { 0, 40, 50, 60, 70, 80, 90, 100, 110, 120 };
    private String[] ar_VerkehrsDatenLzzKlasseBPAV2007 = { "qBPA", "vBPA", "sBPA", "v15BPA", "v85BPA" };

    /** Array mit allen LVo Werten für einen Datensatz BASt-Format 2007. */
    private String[] ar_VerkehrsDatenLzzKlasseLVoV2007 = { "qSV", "qLVo", "vLVo", "sLVo", "v15LVo", "v85LVo" };
    private String[] ar_VerkehrsDatenLzzKlasseSGVV2007 = { "qSGV", "vSGV", "sSGV", "v15SGV", "v85SGV" };

    /** Instanz der Klasse BastBestandsbandformat. */
    private BastBestandsbandformat _bastDatensatz;

    /** Statusinformationen die an den Datenverteiler gesendet werden. */
    private SendeBAStStatusDaten _bastStatusDaten;

    //~ KONSTRUKTOREN  (und vom Konstruktor verwendete Methoden) ==============

    public ImportLineSplitterBASt(BastBestandsbandformat bastDatensatz, SendeBAStStatusDaten bastStatusDaten) {
        _bastDatensatz   = bastDatensatz;
        _bastStatusDaten = bastStatusDaten;
    }

    //~ METHODEN ==============================================================

    /**
     * Die Methode bestimmt die Zeit des Stundendatensatzes.
     *
     * @param date Datum.
     *
     * @return Zeit in millisekunden.
     */
    private long bestimmeStundensatzZeit(Date date) {
        Calendar cal = Calendar.getInstance();

        cal.setTime(date);

        // cal.set(Calendar.HOUR, cal.get(Calendar.HOUR) - 1);
        return cal.getTimeInMillis();
    }

    /**
     * Stundendatensatzwerte (vLVo, vSGV, vBPA, sLVo, sSGV, sBPA, v85LVo, v85SGV, v85BPA, v15LVo, v15SGV, v15BPA,
     * qLVoGeschwKlasse, qSGVGeschwKlasse, qBPAGeschwKlasse) werden eingelesen.
     *
     * @param fahrstreifen  Fahrstreifen.
     * @param teilString    zu parsende Teilstring.
     * @param longDatumZeit Datum.
     * @param line          Zeile.
     */
    private void importiereStundendatensatz(Fahrstreifen fahrstreifen, ParseStundensatzV2007 teilString, long longDatumZeit, String line) {
        HashMap<String, String> mapFahrZeugArtWert = new HashMap<String, String>();

        // start Position 15  ab hier für den ersten Fahrstreifen....
        for (String wert : ar_VerkehrsDatenLzzKlasseLVoV2007) {
            mapFahrZeugArtWert.put(wert, teilString.getFolgewert(teilString.get_cursorStart(), teilString.get_cursorEnd(), line));
        }

        HashMap<Integer, String> geschwKlassenLvo = new HashMap<Integer, String>();

        for (Integer gKlasse : _lVoValues) {
            geschwKlassenLvo.put(gKlasse, teilString.getFolgewert(teilString.get_cursorStart(), teilString.get_cursorEnd(), line));
        }

        for (String wert : ar_VerkehrsDatenLzzKlasseSGVV2007) {
            mapFahrZeugArtWert.put(wert, teilString.getFolgewert(teilString.get_cursorStart(), teilString.get_cursorEnd(), line));
        }

        HashMap<Integer, String> geschwKlassenSgv = new HashMap<Integer, String>();

        for (Integer gKlasse : _sGVValues) {
            geschwKlassenSgv.put(gKlasse, teilString.getFolgewert(teilString.get_cursorStart(), teilString.get_cursorEnd(), line));
        }

        HashMap<Integer, String> geschwKlassenBPA = new HashMap<Integer, String>();

        if (_bastDatensatz.getKlassifikation().getAnzahlFahrzeuggruppen() == 3) {
            for (String wert : ar_VerkehrsDatenLzzKlasseBPAV2007) {
                mapFahrZeugArtWert.put(wert, teilString.getFolgewert(teilString.get_cursorStart(), teilString.get_cursorEnd(), line));
            }

            for (Integer gKlasse : _bPAValues) {
                geschwKlassenBPA.put(gKlasse, teilString.getFolgewert(teilString.get_cursorStart(), teilString.get_cursorEnd(), line));
            }
        }

        Stundendatensatz stundenDatensatz = new Stundendatensatz();

        stundenDatensatz.setDatumZeit(longDatumZeit);
        stundenDatensatz.setMapFZArtVerkehrsMenge(mapFahrZeugArtWert);

        HashMap<String, HashMap<Integer, String>> mapGeschwindigkeitsKlassen = new HashMap<String, HashMap<Integer, String>>();

        mapGeschwindigkeitsKlassen.put(LVO, geschwKlassenLvo);
        mapGeschwindigkeitsKlassen.put(SGV, geschwKlassenSgv);
        mapGeschwindigkeitsKlassen.put(BPA, geschwKlassenBPA);
        stundenDatensatz.setGKlasseVerkehrsMengeGKlasse(mapGeschwindigkeitsKlassen);

        ArrayList<Stundendatensatz> arListStundenDatensaetze = _bastDatensatz.getMapFSArrayListStundendatenSaetze8Plus1().get(fahrstreifen);

        if (arListStundenDatensaetze == null) {
            arListStundenDatensaetze = new ArrayList<Stundendatensatz>();
        }

        arListStundenDatensaetze.add(stundenDatensatz);
        _bastDatensatz.getMapFSArrayListStundendatenSaetze8Plus1().put(fahrstreifen, arListStundenDatensaetze);
    }

    /**
     * Parsing der dritten Header Zeile.
     * <ul>
     * <li> Position 1: Headerkennung 3. Datensatz -- Bsp. S </li>
     * <li> Position 2-3: Anzahl der FAhrzeuggruppen -- Bsp. 02 </li>
     * <li> Position 4: Leerzeichen </li>
     * <li> Position 5-6: Anzahl der Fahrzeugarten -- Bsp. 09 </li>
     * </ul>
     * .... je nach Konfiguration.
     *
     * @param line      Zeile des dirtten Headerdatensatzes.
     * @param dateiName DateiName der gerade einzulesenden Datei.
     *
     * @return true wenn der aktuelle Import abgebrochen werden soll.
     */
    public boolean parseDritteHeaderLine2004(String line, String dateiName) {
        boolean abbruch = false;

        try {
            _bastDatensatz.getKlassifikation().setHeaderkennung(line.substring(0, 1));
            _bastDatensatz.getKlassifikation().setAnzahlFahrzeuggruppen(Integer.parseInt((line.substring(1, 3)).trim()));
            _bastDatensatz.getKlassifikation().setAnzahlFahrzeugarten(Integer.parseInt((line.substring(4, 6)).trim()));
        }
        catch (NumberFormatException e) {
            sendeStatusMeldungUndDebugInfo("Im Importverzeichnis V2004 befindet sich eine Datei " + dateiName + " die nicht eingelesen werden kann, bitte kontrollieren Sie diese Datei.");
            abbruch = true;
        }

        return abbruch;
    }

    /**
     * Parsing der dritten Header Zeile.
     * <ul>
     * <li> Position 1: Headerkennung 3. Datensatz -- Bsp. S </li>
     * <li> Position 2-3: Anzahl der FAhrzeuggruppen -- Bsp. 02 </li>
     * </ul>
     * .... je nach Konfiguration.
     *
     * @param line      Zeile des dirtten Headerdatensatzes.
     * @param dateiName Dateiname der aktuell importierten Datei.
     *
     * @return true wenn der aktuelle Importvorgang abgebrochen werden soll.
     */
    public boolean parseDritteHeaderLine2007(String line, String dateiName) {
        boolean                   abbruch     = false;
        HashMap<Integer, Integer> lvoGKlassen = new HashMap<Integer, Integer>();
        HashMap<Integer, Integer> sgvGKlassen = new HashMap<Integer, Integer>();
        HashMap<Integer, Integer> bpaGKlasse  = new HashMap<Integer, Integer>();

        _bastDatensatz.getKlassifikation().setHeaderkennung(line.substring(0, 1));

        try {
            _bastDatensatz.getKlassifikation().setAnzahlFahrzeuggruppen(Integer.parseInt(line.substring(1, 2)));
        }
        catch (NumberFormatException e) {
            sendeStatusMeldungUndDebugInfo("Im Importverzeichnis V2007 befindet sich eine Datei " + dateiName + " die nicht eingelesen werden kann, bitte kontrollieren Sie diese Datei (Fehler in der dritten Headerzeile).");
            abbruch = true;
        }

        ParseHeaderLine parseHeaderLine = new ParseHeaderLine(55, 58);

        for (int i = 0; i < 16; i++) {
            lvoGKlassen = parseHeaderLine.parseFolgeWert(lvoGKlassen, parseHeaderLine.get_cursorStart(), parseHeaderLine.get_cursorEnd(), line, 0);
        }

        for (int i = 0; i < 10; i++) {
            sgvGKlassen = parseHeaderLine.parseFolgeWert(sgvGKlassen, parseHeaderLine.get_cursorStart(), parseHeaderLine.get_cursorEnd(), line, 0);
        }

        if (Integer.parseInt(line.substring(1, 2)) == 3) {
            for (int i = 0; i < 10; i++) {
                bpaGKlasse = parseHeaderLine.parseFolgeWert(bpaGKlasse, parseHeaderLine.get_cursorStart(), parseHeaderLine.get_cursorEnd(), line, 0);
            }
        }

        return abbruch;
    }

    /**
     * Parsing der ersten Header Zeile.
     * <ul>
     * <li> Position 1: Headerkennung 1. Datensatz -- Bsp. H </li>
     * <li> Position 2-5: Blatt-Nr. der TK25 -- Bsp. 3608 </li>
     * <li> Position 6-9: Zählstellennummer -- Bsp. 3357 </li>
     * <li> Position 10: Leerzeichen </li>
     * <li> Position 11-12: Bundesland: Kennung -- Bsp. 03 </li>
     * <li> Position 13: Leerzeichen </li>
     * <li> Position 14: Straßenklasse -- Bsp. A </li>
     * <li> Position 15: Leerzeichen -- </li>
     * <li> Position 16-20: Straßennummer und ggf. Buchstabe -- Bsp. 30 </li>
     * <li> Position 21: Leerzeichen -- </li>
     * <li> Position 22-46: Zählstellenname -- Bsp. Gildehaus </li>
     * <li> Position 47-50: Versionsnummer der Struktur -- Bsp V2.0 </li>
     * <li> Position 51: Zeilenende -- ; </li>
     * </ul>
     *
     * @param line      Zeile aus der Datei.
     * @param dateiName Dateiname der aktuell eingelesenen Datei.
     *
     * @return LangZeitZaehlStelle.
     */
    public LangZeitZaehlStelle parseErsteHeaderLineV2004(String line, String dateiName) {
        boolean abbruch = false;
        String  lzzNr   = "";
        String  blattNr = "";

        try {
            _bastDatensatz.getZaehlStellenAngaben().setHeaderkennung(line.substring(0, 1));
            blattNr = line.substring(1, 5);
            lzzNr   = line.substring(5, 9);
            _bastDatensatz.getZaehlStellenAngaben().setBlattNr(blattNr);
            _bastDatensatz.getZaehlStellenAngaben().setLzzNr(lzzNr);
            _bastDatensatz.getZaehlStellenAngaben().setBundesLandKennung(line.substring(10, 12));
            _bastDatensatz.getZaehlStellenAngaben().setStrassenKlasse(line.substring(13, 14));
            _bastDatensatz.getZaehlStellenAngaben().setStrassenNr(line.substring(15, 20));
            _bastDatensatz.getZaehlStellenAngaben().setLzzName(line.substring(21, 46));
            _bastDatensatz.getZaehlStellenAngaben().setVersionsNummer(line.substring(46, 50));
        }
        catch (Exception e) {
            sendeStatusMeldungUndDebugInfo("Im Importverzeichnis V2004 befindet sich eine Datei " + dateiName + " die nicht eingelesen werden kann, bitte kontrollieren Sie diese Datei.");
            abbruch = true;
        }

        if (!abbruch) {
            try {
                LangZeitZaehlStelle[] allLZZ = LangZeitZaehlStelle.getAllLve();

                // Suche die passende LZZ über die Zählstellennummer
                // (bestehht aus blattNr und lzzNr) statt nur über lzzNr
                // (da in aktueller Konfiguration BW-DAUZ lzzNr nicht eindeutig sind).
                String lzz = String.format("%s%s", blattNr, lzzNr);

                for (LangZeitZaehlStelle lz : allLZZ) {
                    if (lz.getLzzNummerAsString().equals(lzz)) {
                        _bastDatensatz.getZaehlStellenAngaben().setLzz(lz);

                        return lz;
                    }
                }

                // BW-DAUZ: LZZ-Nummer in der BASt-Datei von BW startet immer mit 8
                // statt 1 ---> ersetze 1.Symbol in der Nummer um die richtige LZZ zu finden:
                String lzzKorrigiert = String.format("%s1%s", blattNr, lzzNr.substring(1));

                for (LangZeitZaehlStelle lz : allLZZ) {
                    if (lz.getLzzNummerAsString().equals(lzzKorrigiert)) {
                        _bastDatensatz.getZaehlStellenAngaben().setLzz(lz);

                        return lz;
                    }
                }
            }
            catch (DataModellMismatchException e) {
                sendeStatusMeldungUndDebugInfo("Die Langzeitzählstellen konnten nicht eingelesen werden. " + e);
            }
        }

        sendeStatusMeldungUndDebugInfo("Im Importverzeichnis V2004 befindet sich eine Datei " + dateiName + " die nicht eingelesen werden kann, bitte kontrollieren Sie diese Datei.");

        return null;
    }

    /**
     * Parsing der ersten Header Zeile.
     * <ul>
     * <li> Position 1-4: Blattnummer der TK25 -- Bsp. 4807 </li>
     * <li> Position 5-8:Zählstellennummer -- Bsp. 50300 </li>
     * <li> Position 9: Leerzeichen </li>
     * <li> Position 10-11: Bundesland: Kennung -- Bsp. 03 </li>
     * <li> Position 12: Leerzeichen </li>
     * <li> Position 13: Straßenklasse -- Bsp. A </li>
     * <li> Position 14: Leerzeichen -- </li>
     * <li> Position 15-20: Straßennummer und ggf. Buchstabe -- Bsp. 30 </li>
     * <li> Position 21: Leerzeichen -- </li>
     * <li> Position 22-61: Zählstellenname -- Bsp. Gildehaus </li>
     * <li> Position 62: Leerzeichen -- </li>
     * <li> Position 63-66: Versionsnummer der Struktur  -- Bsp. V2.0 </li>
     * <li> Position 67: Zeilenende -- ; </li>
     * </ul>
     *
     * @param line      Zeile aus der Datei.
     * @param dateiName Dateiname der aktuell eingelesenen Datei.
     *
     * @return LangZeitZaehlStelle.
     */
    public LangZeitZaehlStelle parseErsteHeaderLineV2007(String line, String dateiName) {
        boolean abbruch  = false;
        String  infoText = "Im Importverzeichnis V2007 befindet sich eine Datei " + dateiName + " die nicht eingelesen werden kann, bitte kontrollieren Sie diese Datei (Fehler in der ersten Headerzeile).";

        try {
            _bastDatensatz.getZaehlStellenAngaben().setBlattNr(line.substring(1, 4));
            _bastDatensatz.getZaehlStellenAngaben().setLzzNr(line.substring(4, 8));
            _bastDatensatz.getZaehlStellenAngaben().setBundesLandKennung(line.substring(9, 11));
            _bastDatensatz.getZaehlStellenAngaben().setStrassenKlasse(line.substring(12, 13));
            _bastDatensatz.getZaehlStellenAngaben().setStrassenNr(line.substring(14, 20));
            _bastDatensatz.getZaehlStellenAngaben().setLzzName(line.substring(21, 61));
            _bastDatensatz.getZaehlStellenAngaben().setVersionsNummer(line.substring(62, 66));
        }
        catch (Exception e) {
            sendeStatusMeldungUndDebugInfo(infoText);
            abbruch = true;
        }

        if (!abbruch) {
            try {
                LangZeitZaehlStelle[] allLZZ = LangZeitZaehlStelle.getAllLve();

                for (LangZeitZaehlStelle lz : allLZZ) {
                    if (lz.getBastNummerAsString().equals(line.substring(4, 8))) {
                        _bastDatensatz.getZaehlStellenAngaben().setLzz(lz);

                        return lz;
                    }
                }
            }
            catch (DataModellMismatchException e) {
                sendeStatusMeldungUndDebugInfo("Die Langzeitzählstellen der Lve konnten nicht eingelesen werden." + e);
            }
        }

        return null;
    }

    /**
     * Die Methode parseStundenDatensaetze2004 ruft je nach Konfiguration die richtige Parsemethode auf, die sich bzgl.
     * der Einzulesenden Daten unterscheidet.
     *
     * @param line Zeile die eingelesen wird.
     *
     * @return map.
     */
    public BastBestandsbandformat parseStundenDatensaetze2004(String line) {
        if (_bastDatensatz.getKlassifikation().isKlassifizierung1()) {

            // nicht implementiert.
        } else if (_bastDatensatz.getKlassifikation().isKlassifizierung2()) {
            return parseStundenDatensaetze2004(line, AR_VERKEHRSDATEN_LZZ_2_V2004_GRUPPEN, AR_VERKEHRSDATEN_LZZ_2_V2004_ARTEN);
        } else if (_bastDatensatz.getKlassifikation().isKlassifizierung5Plus1()) {
            return parseStundenDatensaetze2004(line, AR_VERKEHRSDATEN_LZZ_5_PLUS_1_V2004_GRUPPEN, AR_VERKEHRSDATEN_LZZ_5_PLUS_1_V2004_ARTEN);
        } else if (_bastDatensatz.getKlassifikation().isKlassifizierung8Plus1()) {
            return parseStundenDatensaetze2004(line, AR_VERKEHRSDATEN_LZZ_8_PLUS_1_V2004_GRUPPEN, AR_VERKEHRSDATEN_LZZ_8_PLUS_1_V2004_ARTEN);
        }

        return null;
    }

    /**
     * Die Methode liest die Stundendatensätze entsprechend es Formates 2, 5Plus1 oder 8Plus1 ein.
     * Start bei Position 13
     *
     * <li> Position 14-17: Anzahl KFZ auf FS1 Richtung I </li>
     * <li> Position 18: Kennzeichen KFZ auf FS1 Richtung 1</li>
     * <li> Position 19: Leerzeichen </li>
     * <li> Position 20-23: Anzahl SV auf FS1 Richtung I </li>
     * <li> Position 24: Kennzeichen SV auf FS1 Richtung I </li>
     * <li> Position 25: Leerzeichen </li> ....
     * s. BASt.Besatandsformat_Version2004 Für jeden Fahrstreifen Verkehrsmengen der Fahrzeuggruppen wie folgt für 8Plus1
     * bestimmt: FS i R1: FZGruppe I: Son + Mot + Pkw + Lfw + PmA       (Attribut _fsGruppe1R1) FS i R1: FZGruppe II:
     * LoA + LmA + Sat + Bus            (Attribut _fsGruppe2R1) FS i+1 R1: analog FS i+1 R1: analog ... FS i+n R2:
     * FZGruppe I: Son + Mot + Pkw + Lfw + PmA Fs i+n R2: FZGruppe II: LoA + LmA + Sat + Bus ... Fs i R2: FZGruppe I:
     * Son + Mot + Pkw + Lfw + PmA    (Attribut _fsGruppe1R2) Fs i R2: FZGruppe II: LoA + LmA + Sat + Bus
     * (Attribut _fsGruppe2R2)
     * <p>
     * Reihenfolge der  auszulesenden Datensätze: Mot       Pkw      LfW      PmA       Bus        LoA     LmA       Sat
     *             Son Krad       Pkw      Lfw      PkwA      Bus        Lkw     LkwA      Sattel-Kfz      nkKfz "qKrad"
     *    "qPkw"   "qLfw"   "qPkwA"   "qBus"    "qLkw"   "qLkwA"   "qSattelKfz"   "qKfzNk" (Map mit Schlüssel
     * Fahrzeugart, Wert VerkehrsMenge _mapFZArtVerkehrsMege, wobei die Werte direkt den DaV Werten entsprechen, d.h.
     * z.B. (qKrad, 0) entspricht dann einem Eintrag in den Stundensätzen Mot. Dafür wird eine Schlüsselmap verwendet,
     * die die unterschiedlichen Bezeichnungen TLS2002 <-> BASt-Bezeichnung aufeinander abbildet.) "qKfz", "qPkwÄ",
     * "qPkwG",  "qLkwÄ",  "qLkwK",
     *
     * @param line            Zeile die ausgelesen wird.
     * @param arVerkehrsDatenGruppen Array mit den Verkehrsdaten (Fahrzeuggruppen).
     * @param arVerkehrsDatenArten Array mit den Verkehrsdaten (Fahrzeugarten).
     *
     * @return Map mit allen ermittelten Informationen aus der Datei zu dem Fahrstreifen.
     *         <p>
     *         int iAnzahlKFZ =   Integer.parseInt(line.substring(14, 17));
     */
    private BastBestandsbandformat parseStundenDatensaetze2004(String line, String[] arVerkehrsDatenGruppen, String[] arVerkehrsDatenArten) {
        Fahrstreifen[]   fsInAuswertequerschnittRichtung    = _bastDatensatz.getZaehlStellenAngaben().getLzz().getInAuswertequerschnitt().getMessQuerschnitt().getFahrstreifen();
        Fahrstreifen[]   fsGegenAuswertequerschnittRichtung = _bastDatensatz.getZaehlStellenAngaben().getLzz().getGegenAuswertequerschnitt().getMessQuerschnitt().getFahrstreifen();
        String[][]       arGruppenIn                        = new String[arVerkehrsDatenGruppen.length][fsInAuswertequerschnittRichtung.length];
        String[][]       arGruppenGegen                     = new String[arVerkehrsDatenGruppen.length][fsGegenAuswertequerschnittRichtung.length];
        long             longDatumZeit                      = 0;
        SimpleDateFormat datum                              = new SimpleDateFormat("yyMMdd kk:mm");

        // Der cursur repräsentiert die aktuelle Zeilenposition.
        ParseSpaltenWert spaltenWert = new ParseSpaltenWert(12);

        try {
            longDatumZeit = bestimmeStundensatzZeit(datum.parse(line.substring(0, 12).replaceAll("[a-zA-Z]", " ")));

            // Weil Bast von 1 Uhr bis 24 Uhr darstellt, müssen Zeitstempel um eine Stunde korrigiert werden
            // von 0 bis 23 Uhr.
            // ACHTUNG: Die Stundenangabe 010510 24:00 wird beim Parsen auf 010510 00:00 umgesetzt!!
            // Daraus folgt: Wenn Stunde != 24 dann 1 Stunde  abziehen, sonst 23 Stunden adieren!
            Calendar cal = Calendar.getInstance();

            cal.setTimeInMillis(longDatumZeit);

            if (cal.get(Calendar.HOUR_OF_DAY) == 0) {
                cal.add(Calendar.HOUR_OF_DAY, 23);
            } else {
                cal.add(Calendar.HOUR_OF_DAY, -1);
            }

            longDatumZeit = cal.getTimeInMillis();
        }
        catch (ParseException e) {
            debug.error("Es sind Probleme beim Parsten des Datums aufgetreten, bitte kontrollieren Sie die Eingabedatei." + e, e);
        }

        // Verkehrsmengen und Prüfkennzeichen der FZGruppen, vollständig für alle FS der einen Richtung,
        // dann vollständig für die andere Richtung.
        for (int iInRichtung = 0; iInRichtung < fsInAuswertequerschnittRichtung.length; iInRichtung++) {
            for (int iGruppe = 0; iGruppe < arVerkehrsDatenGruppen.length; iGruppe++) {
                arGruppenIn[iGruppe][iInRichtung] = spaltenWert.parseFolgeWert(line);
            }
        }

        for (int iGegenRichtung = 0; iGegenRichtung < fsGegenAuswertequerschnittRichtung.length; iGegenRichtung++) {
            for (int iGruppe = 0; iGruppe < arVerkehrsDatenGruppen.length; iGruppe++) {
                arGruppenGegen[iGruppe][iGegenRichtung] = spaltenWert.parseFolgeWert(line);
            }
        }

        int inR = 0;

        for (Fahrstreifen fsIn : fsInAuswertequerschnittRichtung) {
            HashMap<String, String> mapFahrZeugArtWert = new HashMap<String, String>();

            // Bestimme pro FS Richtung I und Pro FS Richtung II  alle Werte.
            for (int iGruppe = 0; iGruppe < arVerkehrsDatenGruppen.length; iGruppe++) {
                String gruppe = arVerkehrsDatenGruppen[iGruppe];

                if (!gruppe.equals(ATTRIBUT_UNDEFINIERT)) {
                    mapFahrZeugArtWert.put(gruppe, arGruppenIn[iGruppe][inR]);
                }
            }

            inR++;

            for (String wert : arVerkehrsDatenArten) {
                mapFahrZeugArtWert.put(wert, spaltenWert.parseFolgeWert(line));
            }

            Stundendatensatz stundenDatensatz = new Stundendatensatz();

            stundenDatensatz.setDatumZeit(longDatumZeit);
            stundenDatensatz.setMapFZArtVerkehrsMenge(mapFahrZeugArtWert);

            ArrayList<Stundendatensatz> arListStundenDatensaetze = _bastDatensatz.getMapFSArrayListStundendatenSaetze8Plus1().get(fsIn);

            if (arListStundenDatensaetze == null) {
                arListStundenDatensaetze = new ArrayList<Stundendatensatz>();
            }

            arListStundenDatensaetze.add(stundenDatensatz);
            _bastDatensatz.getMapFSArrayListStundendatenSaetze8Plus1().put(fsIn, arListStundenDatensaetze);
        }

        Fahrstreifen[] arFahrstreifen = new Fahrstreifen[fsGegenAuswertequerschnittRichtung.length];
        int            i              = 0;

        for (Fahrstreifen fs : fsGegenAuswertequerschnittRichtung) {
            arFahrstreifen[i] = fs;
            i++;
        }

        int gegenR = 0;

        for (int j = arFahrstreifen.length - 1; j >= 0; j--) {
            HashMap<String, String> mapFahrZeugArtWert = new HashMap<String, String>();

            for (int iGruppe = 0; iGruppe < arVerkehrsDatenGruppen.length; iGruppe++) {
                String gruppe = arVerkehrsDatenGruppen[iGruppe];

                if (!gruppe.equals(ATTRIBUT_UNDEFINIERT)) {
                    mapFahrZeugArtWert.put(gruppe, arGruppenGegen[iGruppe][gegenR]);
                }
            }

            gegenR++;

            // Bestimme pro FS Richtung I und Pro FS Richtung II  alle Werte.
            for (String wert : arVerkehrsDatenArten) {
                mapFahrZeugArtWert.put(wert, spaltenWert.parseFolgeWert(line));
            }

            Stundendatensatz stundenDatensatz = new Stundendatensatz();

            stundenDatensatz.setDatumZeit(longDatumZeit);
            stundenDatensatz.setMapFZArtVerkehrsMenge(mapFahrZeugArtWert);

            ArrayList<Stundendatensatz> arListStundenDatensaetze = _bastDatensatz.getMapFSArrayListStundendatenSaetze8Plus1().get(arFahrstreifen[j]);

            if (arListStundenDatensaetze == null) {
                arListStundenDatensaetze = new ArrayList<Stundendatensatz>();
            }

            arListStundenDatensaetze.add(stundenDatensatz);
            _bastDatensatz.getMapFSArrayListStundendatenSaetze8Plus1().put(arFahrstreifen[j], arListStundenDatensaetze);
        }

        return _bastDatensatz;
    }

    /**
     * Die Methode liest die Stundendatensatzdaten ein.
     *
     * @param line Zeile die eingelesen wird.
     *
     * @return BastBestandsbandformat.
     */
    public BastBestandsbandformat parseStundenDatensaetzeKlassifizierungV2007(String line) {
        Fahrstreifen[]        fsInAuswertequerschnittRichtung    = _bastDatensatz.getZaehlStellenAngaben().getLzz().getInAuswertequerschnitt().getMessQuerschnitt().getFahrstreifen();
        Fahrstreifen[]        fsGegenAuswertequerschnittRichtung = _bastDatensatz.getZaehlStellenAngaben().getLzz().getGegenAuswertequerschnitt().getMessQuerschnitt().getFahrstreifen();
        long                  longDatumZeit                      = 0;
        ParseStundensatzV2007 teilString                         = new ParseStundensatzV2007(15, 20);
        SimpleDateFormat      datum                              = new SimpleDateFormat("yyMMdd kk:mm");

        try {
            longDatumZeit = bestimmeStundensatzZeit(datum.parse(line.substring(0, 12).replaceAll("[a-zA-Z]", " ")));

            // Weil Bast von 1 Uhr bis 24 Uhr darstellt, müssen Zeitstempel um eine Stunde korrigiert werden
            // von 0 bis 23 Uhr.
            // ACHTUNG: Die Stundenangabe 010510 24:00 wird beim Parsen auf 010510 00:00 umgesetzt!!
            // Daraus folgt: Wenn Stunde != 24 dann 1 Stunde  abziehen, sonst 23 Stunden adieren!
            Calendar cal = Calendar.getInstance();

            cal.setTimeInMillis(longDatumZeit);

            if (cal.get(Calendar.HOUR_OF_DAY) == 0) {
                cal.add(Calendar.HOUR_OF_DAY, 23);
            } else {
                cal.add(Calendar.HOUR_OF_DAY, -1);
            }

            longDatumZeit = cal.getTimeInMillis();
        }
        catch (ParseException e) {
            sendeStatusMeldungUndDebugInfo("Es sind Probleme beim Parsen des Datums aufgetreten. Bitte kontrollieren Sie die Eingabedatei." + e);
        }

        int richtung = 0;

        try {
            richtung = Integer.parseInt(line.substring(13, 14));
        }
        catch (NumberFormatException e) {
            sendeStatusMeldungUndDebugInfo("Probleme beim Parsen der Richtungsangabe. Bitte kontrollieren Sie die Eingabedatei." + e);

            try {
                _bastStatusDaten.sendeBAStStatusDaten("", "Die Richtungsangaben in dem" + "zu importierendem Dokument können nicht eingelesen werden, der Import wird abgebrochen, die" + "Sperre der Datei bleibt erhalten.");
            }
            catch (SendSubscriptionNotConfirmed sendSubscriptionNotConfirmed) {
                sendeStatusMeldungUndDebugInfo("Es sind Probleme beim Senden der Daten aufgetreten." + sendSubscriptionNotConfirmed);
            }
        }

        if (richtung == 1) {
            for (Fahrstreifen fsIn : fsInAuswertequerschnittRichtung) {
                importiereStundendatensatz(fsIn, teilString, longDatumZeit, line);
            }
        } else if (richtung == 2) {
            for (int i = fsGegenAuswertequerschnittRichtung.length - 1; i >= 0; i--) {
                importiereStundendatensatz(fsGegenAuswertequerschnittRichtung[i], teilString, longDatumZeit, line);
            }
        } else {
            try {
                _bastStatusDaten.sendeBAStStatusDaten("", "Die Richtungsangabe ist" + "weder in Richtung 1 noch in Richtung 2, bitte überprüfen Sie dies in dem zu importierendem Dokument. Der" + "Import wird abgebrochen, die Sperre der Datei bleibt erhalten.");
            }
            catch (SendSubscriptionNotConfirmed sendSubscriptionNotConfirmed) {
                sendeStatusMeldungUndDebugInfo("Es sind Probleme beim Senden der Daten aufgetreten." + sendSubscriptionNotConfirmed);
            }
        }

        return _bastDatensatz;
    }

    /**
     * Parsing der zweiten Header Zeile.
     * <ul>
     * <li> Position 1: Headerkennung 2. Datensatz -- Bsp. R </li>
     * <li> Position 2-3: Anzahl der Fahrstreifen in Richtung I -- Bsp. 02 </li>
     * <li> Position 4: Leerzeichen </li>
     * <li> Position 5-6: Anzahl der Fahrstreifen in Richtung II -- Bsp. 02 </li>
     * <li> Position 7: Leerzeichen </li>
     * <li> Position 8-27: Fernziel in Richtung I -- Bsp. Gildehaus </li>
     * <li> Position 28: Himmelsrichtung I -- Bsp. O </li>
     * <li> Position 29: Leerzeichen </li>
     * <li> Position 30-49: Fernziel in Richtung II -- Bsp. Hengelo (NL) </li>
     * <li> Position 50: Himmelsrichtung II -- Bsp. W </li>
     * <li> Position 51: Zeilenende </li>
     * </ul>
     *
     * @param line      Zeile des zweiten Headerdatensatzes.
     * @param dateiName DateiName der aktuell einzulesenden Datei.
     *
     * @return true wenn die aktuelle Datei nicht eingelesen werden kann.
     */
    public boolean parseZweiteHeaderLine2004(String line, String dateiName) {
        boolean abbruch = false;

        try {
            _bastDatensatz.getRichtungsAngaben().setHeaderkennung(line.substring(0, 1));
            _bastDatensatz.getRichtungsAngaben().setAnzahlFSRichtungI(line.substring(1, 3));
            _bastDatensatz.getRichtungsAngaben().setAnzahlFSRichtungII(line.substring(4, 6));
            _bastDatensatz.getRichtungsAngaben().setFernzielRichtungI(line.substring(7, 27));
            _bastDatensatz.getRichtungsAngaben().setHimmelsrichtungI(line.substring(27, 28));
            _bastDatensatz.getRichtungsAngaben().setFernzielRichtungII(line.substring(29, 49));
            _bastDatensatz.getRichtungsAngaben().setHimmelsrichtungII(line.substring(49, 50));
        }
        catch (Exception e) {
            sendeStatusMeldungUndDebugInfo("Im Importverzeichnis V2004 befindet sich eine Datei " + dateiName + " die nicht eingelesen werden kann, bitte kontrollieren Sie diese Datei.");
            abbruch = true;
        }

        return abbruch;
    }

    /**
     * Parsing der zweiten Header Zeile.
     * <ul>
     * <li> Position 1: Anzahl der Fahrstreifen in Richtung I -- Bsp. 02 </li>
     * <li> Position 2: Leerzeichen </li>
     * <li> Position 3: Anzahl der Fahrstreifen in Richtung II -- Bsp. 02 </li>
     * <li> Position 4: Leerzeichen </li>
     * <li> Position 5-39: Fernziel in Richtung I -- Bsp. Oberhausen </li>
     * <li> Position 40: Leerzeichen </li>
     * <li> Position 41-85: Nahziel in Richtung I  -- AS Solingen  </li>
     * <li> Position 86: Leerzeichen </li>
     * <li> Position 87-121: Fernziel in Richtung II -- Bsp. Köln </li>
     * <li> Position 122: Leerzeichen </li>
     * <li> Position 123-167: Nahziel in Richtung II -- Bsp. AS Opladen </li>
     * <li> Position 168: Zeilenende -- ; </li>
     * </ul>
     *
     * @param line      Zeile des zweiten Headerdatensatzes.
     * @param dateiName DateiName
     *
     * @return true wenn die Datei nicht eingelesen werden kann.
     */
    public boolean parseZweiteHeaderLine2007(String line, String dateiName) {
        boolean abbruch = false;

        try {
            _bastDatensatz.getRichtungsAngaben().setAnzahlFSRichtungI(line.substring(0, 1));
            _bastDatensatz.getRichtungsAngaben().setAnzahlFSRichtungII(line.substring(1, 2));
            _bastDatensatz.getRichtungsAngaben().setFernzielRichtungI(line.substring(4, 39));
            _bastDatensatz.getRichtungsAngaben().setNahzielRichtungI(line.substring(40, 85));
            _bastDatensatz.getRichtungsAngaben().setFernzielRichtungII(line.substring(86, 121));
            _bastDatensatz.getRichtungsAngaben().setNahzielRichtungII(line.substring(122, 167));
        }
        catch (Exception e) {
            sendeStatusMeldungUndDebugInfo("Im Importverzeichnis V2007 befindet sich eine Datei " + dateiName + " die nicht eingelesen werden kann, bitte kontrollieren Sie diese Datei (Fehler in der zweiten Headerzeile).");
            abbruch = true;
        }

        return abbruch;
    }

    /**
     * Die Methode sendet die Statusmeldung und via Debug eine Information.
     *
     * @param statusMeldungUndDebugInfo Meldung als String.
     */
    private void sendeStatusMeldungUndDebugInfo(String statusMeldungUndDebugInfo) {
        debug.info(statusMeldungUndDebugInfo);

        try {
            _bastStatusDaten.sendeBAStStatusDaten("", statusMeldungUndDebugInfo);
        }
        catch (SendSubscriptionNotConfirmed sendSubscriptionNotConfirmed) {
            debug.info("Problemen beim Senden der Statusinformationsdaten. " + sendSubscriptionNotConfirmed);
        }
        catch (Exception e) {
            debug.info("Problemen beim Senden der Statusinformationsdaten. ", e);
        }
    }
}


class ParseHeaderLine {
    private int _cursorEnd   = 0;
    private int _cursorStart = 0;

    //~ KONSTRUKTOREN  (und vom Konstruktor verwendete Methoden) ==============

    public ParseHeaderLine(int cursorStart, int cursorEnd) {
        _cursorStart = cursorStart;
        _cursorEnd   = cursorEnd;
    }

    //~ METHODEN ==============================================================

    public int get_cursorEnd() {
        return _cursorEnd;
    }

    public int get_cursorStart() {
        return _cursorStart;
    }

    public HashMap<Integer, Integer> parseFolgeWert(HashMap<Integer, Integer> map, int cursorStart, int cursorEnd, String line, int value) {
        if (cursorEnd < line.length()) {
            String wert        = line.substring(cursorStart, cursorEnd).trim();
            int    integerWert = 0;

            try {
                integerWert = Integer.parseInt(wert);
            }
            catch (NumberFormatException e) {
                e.printStackTrace();
            }

            map.put(integerWert, value);
            _cursorStart = cursorStart + 4;
            _cursorEnd   = cursorEnd + 4;
        } else {}

        return map;
    }
}


/** Die Klasse ermittelt den folgenden Stundendatensatz. */
class ParseSpaltenWert {
    private static int[]        AR_KORREKTE_PRUEFKENNZEICHEN = { '-', 'u', 's', 'a', 'd', 'k', 'z' };
    private static final String NICHT_ERMITTELBAR            = "nicht ermittelbar/fehlerhaft";

    //~ FELDER ================================================================

    private int _cursorEnd = 0;

    //~ KONSTRUKTOREN  (und vom Konstruktor verwendete Methoden) ==============

    public ParseSpaltenWert(int cursorEnd) {
        _cursorEnd = cursorEnd;
    }

    //~ METHODEN ==============================================================

    /**
     * Parse folgenden (ab der Kursor-Position) Wert aus der Zeile.
     * Kursor wird dabei an das Ende des Wertes verschoben.
     *
     * Für das Parsen muss der Wert mit einem der korrekten Prüfkennzeichen beendet werden.
     * Andersfall wird NICHT_ERMITTELBAR zurückgeliefert.
     *
     * @param line Zeile.
     *
     * @return Folgender (ab der Kursor-Position) Wert aus der Zeile.
     * Wert wird ohne Prüfkennzeichen ermittelt.
     *
     */
    public String parseFolgeWert(String line) {

        // Merke die Startposition:
        int cursorStart = _cursorEnd;

        if (cursorStart < line.length()) {
            int cursorKorrekteWerteMin = Integer.MAX_VALUE;

            // Suche ein der korrekten Prüfkennzeichen ab der Kursor-Position:
            for (int kennzeichen : AR_KORREKTE_PRUEFKENNZEICHEN) {
                int cursorKorrekteWerte = line.indexOf(kennzeichen, cursorStart + 1);

                if ((cursorKorrekteWerte > -1)
                    && (cursorKorrekteWerte < cursorKorrekteWerteMin)) {
                    cursorKorrekteWerteMin = cursorKorrekteWerte;
                }
            }

            // Falls ein korrekter Prüfkennzeichen gefunden wurde:
            if (cursorKorrekteWerteMin < Integer.MAX_VALUE) {

                // Verschiebe Kursor:
                _cursorEnd = cursorKorrekteWerteMin;

                // Wert zwischen Kursor-Anfang und Prüfkennzeichen:
                return line.substring(cursorStart + 1, _cursorEnd).trim();
            }

            // Kein korrekter Prüfkennzeichen:
            else {
                _cursorEnd = line.length();

                return NICHT_ERMITTELBAR;
            }
        }

        // Falls Kursor nach der Zeile steht:
        else {
            return "";
        }
    }
}


class ParseStundensatzV2007 {
    private int _cursorEnd   = 0;
    private int _cursorStart = 0;

    //~ KONSTRUKTOREN  (und vom Konstruktor verwendete Methoden) ==============

    public ParseStundensatzV2007(int cursorStart, int cursorEnd) {
        _cursorStart = cursorStart;
        _cursorEnd   = cursorEnd;
    }

    //~ METHODEN ==============================================================

    public int get_cursorEnd() {
        return _cursorEnd;
    }

    public int get_cursorStart() {
        return _cursorStart;
    }

    //~ GET METHODEN ==========================================================

    public String getFolgewert(int cursorStart, int cursorEnd, String line) {
        if (cursorEnd < line.length()) {
            _cursorStart = cursorStart + 6;
            _cursorEnd   = cursorEnd + 6;

            return line.substring(cursorStart, cursorEnd).trim();
        }

        return "nicht ermittelbar";
    }
}


//~Formatiert mit 'inovat Kodierkonvention' am 14.02.17
