/*
 * Copyright (c) 2010-2012 by inovat, innovative systeme - verkehr - tunnel - technik, Dipl.-Ing. H. C. Kniss
 *
 * This file is part of de.inovat.kex.bast.serverfunc.FTPFunktionen
 *
 * de.inovat.kex.bast.serverfunc.FTPFunktionen is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * de.inovat.kex.bast.serverfunc.FTPFunktionen is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * <p>
 * You should have received a copy of the GNU General Public License
 * along with de.inovat.kex.bast.serverfunc.FTPFunktionen; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * Contact Information:
 * inovat, Dipl.-Ing. H. C. Kniss
 * Koelner Strasse 30
 * D-50859 Koeln
 * +49 (0)2234 4301 800
 * info@invat.de
 * www.inovat.de
 */



package de.inovat.kex.bast.serverfunc;

//~ NICHT JDK IMPORTE =========================================================

import org.apache.commons.net.ftp.FTPClient;

import java.io.*;

//~ JDK IMPORTE ===============================================================

//~ KLASSEN ===================================================================

/**
 * Klasse stellt statischen FTP-Funktionen zu Verfügung.
 *
 * @author inovat, innovative systeme - verkehr - tunnel - technik
 * @author Liliya Givorgizova (LG)
 * @version $Revision$ / $Date$ / ($Author$)
 */
public class FTPFunktionen {

    /**
     * Trenne die Verbindung mit dem FTP-Server.
     *
     * @param ftpClient FTP-Verbindung.
     *
     * @return Exception, falls bei der Trennung _Fehler aufgetreten sind,
     * null - sonst.
     */
    public static Exception trenneVerbindungMitFTPServer(FTPClient ftpClient) {
        try {
            ftpClient.logout();
            ftpClient.disconnect();

            return null;
        }
        catch (Exception e) {
            return new Exception("Die Verbindung zum FTP-Server konnte nicht beendet werden", e);
        }
    }

    /**
     * Übertrage die FTP-Datei in lokale Datei.
     *
     * @param ftpClient FTP-Verbindung.
     * @param dateiFtp Die zu übertragende FTP-Datei.
     * @param dateiLokal Lokale Datei, wohin die FTP-Datei kopiert werden soll.
     * @param fileTyp Typ der zu übertragenden Datei (s. Konstanten in der Klasse {@link org.apache.commons.net.ftp.FTP}).
     *
     * @return Exception, falls beim Übertragen _Fehler aufgetreten sind,
     * null - sonst.
     */
    public static Exception uebertrageFTPDateiInLokaleDatei(FTPClient ftpClient, String dateiFtp, String dateiLokal, int fileTyp) {
        OutputStream localOutput = null;

        try {

            // Übertrage die FTP-Datei in lokeler Datei:
            localOutput = new FileOutputStream(dateiLokal);

            //
            ftpClient.setFileType(fileTyp);

            boolean istOk = ftpClient.retrieveFile(dateiFtp, localOutput);

            localOutput.close();

            if (istOk) {

                // Prüfe die Größen:
                long sizeLokal = new File(dateiLokal).length();
                long sizeFtp   = ftpClient.listFiles(dateiFtp)[0].getSize();

                if (sizeFtp == sizeLokal) {
                    return null;
                } else {
                    return new Exception(String.format("Die übertragene lokale Datei <%s> hat falsche Größe (%d statt %d)", dateiLokal, sizeLokal, sizeFtp));
                }
            } else {
                return new Exception(String.format("FTP-Datei <%s> konnte nicht in <%s> übertragen werden", dateiFtp, dateiLokal));
            }
        }
        catch (Exception e) {

            // Versuche die Datei zu schließen:
            try {
                if (localOutput != null) {
                    localOutput.close();
                }
            }
            catch (Exception e2) {
                e2.printStackTrace();
            }

            return e;
        }
    }

    /**
     * Übertrage lokale Datei auf FTP-Server.
     * <p>
     * Lokale Datei wird zuerst in FTP-Lock-Datei übertragen.
     * Falls die Größen der lokalen und übertragenen Dateien gleich sind,
     * wird die FTP-Datei umbenannt.
     *
     * @param dateiLokal Die lokale zu übertragende Datei.
     * @param ftpClient FTP-Verbindung.
     * @param dateiFtp FTP-Datei als Ergebnis der Übertragung.
     * @param dateiFtpLock FTP-Lock-Datei, wohin die lokale Datei kopiert werden soll.
     * @param fileTyp Typ der zu übertragenden Datei (s. Konstanten in der Klasse {@link org.apache.commons.net.ftp.FTP}).
     *
     * @return Exception, falls beim Übertragen _Fehler aufgetreten sind,
     * null - sonst.
     */
    public static Exception uebertrageLokaleDateiAufFTPServer(File dateiLokal, FTPClient ftpClient, String dateiFtp, String dateiFtpLock, int fileTyp) {
        InputStream localInput = null;

        try {

            // ---------------------------------------------------------
            // Übertrage lokale Datei auf FTPServer in FTP-Lock-Datei:
            localInput = new FileInputStream(dateiLokal);

            //
            ftpClient.setFileType(fileTyp);

            boolean istOk = ftpClient.storeFile(dateiFtpLock, localInput);

            localInput.close();

            //
            if (istOk) {

                // ---------------------------------------------------------
                // Falls die Größen der lokalen und übertragen Dateien gleich sind,
                // umbenenne die Datei auf dem FTP-Server:
                long sizeLokal = dateiLokal.length();
                long sizeFtp   = ftpClient.listFiles(dateiFtpLock)[0].getSize();

                if (sizeFtp == sizeLokal) {
                    if (ftpClient.rename(dateiFtpLock, dateiFtp)) {
                        return null;
                    } else {
                        return new Exception(String.format("FTP-Datei <%s> konnte nicht in <%s> umbennant werden", dateiFtpLock, dateiFtp));
                    }
                } else {
                    return new Exception(String.format("Die übertragene FTP-Datei <%s> hat falsche Größe (%d statt %d)", dateiFtpLock, sizeFtp, sizeLokal));
                }
            } else {
                return new Exception(String.format("Datei <%s> konnte nicht auf FTP-Server in <%s> übertragen werden", dateiLokal.getAbsolutePath(), dateiFtpLock));
            }
        }
        catch (Exception e) {

            // Versuche die Datei zu schließen:
            try {
                if (localInput != null) {
                    localInput.close();
                }
            }
            catch (Exception e2) {
                e2.printStackTrace();
            }

            return e;
        }
    }

    /**
     * Stelle die Verbindung mit dem FTP-Server her.
     *
     * @param ftpClient FTP-Verbindung.
     * @param url FTP-Host.
     * @param benutzer FTP-Benutzer.
     * @param passwort FTP-Passwort
     *
     * @return Exception, falls bei der Verbindung _Fehler aufgetreten sind,
     * null - sonst.
     */
    public static Exception verbindeMitFTPServer(FTPClient ftpClient, String url, String benutzer, String passwort) {
        return verbindeMitFTPServer(ftpClient, url, 21, benutzer, passwort);
    }

    /**
     * Stelle die Verbindung mit dem FTP-Server her.
     *
     * @param ftpClient FTP-Verbindung.
     * @param url FTP-Host.
     * @param port FTP-Port.
     * @param benutzer FTP-Benutzer.
     * @param passwort FTP-Passwort
     *
     * @return Exception, falls bei der Verbindung _Fehler aufgetreten sind,
     * null - sonst.
     */
    public static Exception verbindeMitFTPServer(FTPClient ftpClient, String url, int port, String benutzer, String passwort) {

        // Verbindung:
        try {
            ftpClient.connect(url, port);
        }
        catch (IOException e) {
            return new Exception(String.format("Es sind Probleme bei der Verbindung mit dem FTP-Server <%s:%s> aufgetreten.", url, port), e);
        }

        // Login:
        String fehlerMeldung = String.format("Der login für <%s> auf dem FTP-Server <%s> ist gescheitert. Bitte kontrollieren Sie den angegebenen Benutzernamen und das Passwort!", benutzer, url);

        try {
            if (ftpClient.login(benutzer, passwort)) {
                return null;
            } else {
                ftpClient.disconnect();

                return new Exception(fehlerMeldung);
            }
        }
        catch (IOException e) {
            return new Exception(fehlerMeldung, e);
        }
    }
}


//~Formatiert mit 'inovat Kodierkonvention' am 18.02.13
